/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: conf.c,v 1.23 2000/11/12 16:05:13 wcampbel Exp $
*/

#include "stats.h"

/* Remove CR and LF from the line input.  ASSUME that the line 
** does NOT contain CR or LF in the middle
*/
void strip(char *line)
{
	char *c;

	if ((c = strchr(line, '\n')))	*c = '\0';
	if ((c = strchr(line, '\r')))	*c = '\0';
}

/* Set default parameters.  The settings used are the same as
** what is provided in stats.cfg.dist except for the binary
** ones, which MUST be default 0 in order for the conf load
** to work properly
*/
void InitDefaults()
{
	strcpy(me.name, "stats.network.tld");
	me.port = 6666;
	strcpy(me.uplink, "hub.network.tld");
	me.use_vhost = 0;
	strcpy(me.vhost, "0.0.0.0");
	strcpy(me.pass, "password");
	strcpy(me.desc, "Network Statistical Services");
	me.r_time = 10;
	me.flood = 6;
	me.flood_time = 10;
	me.flood_action = 0;
	strcpy(StatServ.host, "stats.network.tld");
	strcpy(StatServ.user, "stats");
	me.enable_spam = 0;
	me.onlyopers = 0;
	me.enable_jupe = 0;
	me.enable_quote = 0;
	me.want_privmsg = 0;
	me.hide_services = 0;
	strcpy(me.services_name, "services.network.tld");
	me.send_extreme_lag_notices = 1;
	me.lag_time = 45;
	me.enable_html = 0;
	me.set_oper = 0;
	me.set_deaf = 0;
	strcpy(me.html, "/home/user/public_html/stats.html");
	me.html_time = 3600;
	strcpy(me.html_link, "See our main page <A HREF=\"http://www.network.tld\">here</A>");
	strcpy(me.join_chans, "#statserv,#opers");
	strcpy(me.url, "http://www.network.tld/stats.html");
	strcpy(s_StatServ, "StatServ");
}

/* Load the conf file 
** Some of these should be modified to allow multiple lines...
** SERVICES_NAME to allow multiple servers to be hidden
** CHANNEL and HTML_LINK as well
*/
void ConfLoad()
{
	FILE *fp;
	char buf[BUFSIZE];
	char *s;

	if ((fp = fopen(CONFIG_NAME, "r")) == NULL) {
		printf("Unable to open configuration file (%s)\n", CONFIG_NAME);
		exit(0);
	}

	while (fgets(buf, BUFSIZE, fp)) {
		if (buf[0] == '#')
			continue;
		s = strtok(buf, " ");
		if (!strcmp(s, "SERVER_NAME")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.name, s, sizeof(me.name));
		} else if (!strcmp(s, "SERVER_PORT")) {
			s = strtok(NULL, "\n");
			me.port = atoi(s);
		} else if (!strcmp(s, "CONNECT_TO")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.uplink, s, sizeof(me.uplink));
		} else if (!strcmp(s, "CONNECT_PASS")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.pass, s, sizeof(me.pass));
		} else if (!strncmp(s, "USE_VHOST", 9)) {
			me.use_vhost = 1;
		} else if (!strcmp(s, "VHOST")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.vhost, s, sizeof(me.vhost));
		} else if (!strcmp(s, "SERVER_DESC")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.desc, s, sizeof(me.desc));
		} else if (!strcmp(s, "RECONNECT_TIME")) {
			s = strtok(NULL, "\n");
			me.r_time = atoi(s);
		} else if (!strcmp(s, "FLOOD")) {
			s = strtok(NULL, "\n");
			me.flood = atoi(s);
		} else if (!strcmp(s, "FLOOD_TIME")) {
			s = strtok(NULL, "\n");
			me.flood_time = atoi(s);
		} else if (!strncmp(s, "FLOOD_IGNORE", 12)) {
			me.flood_action = 1;
		} else if (!strcmp(s, "LAG_TIME")) {
			s = strtok(NULL, "\n");
			me.lag_time = atoi(s);
		} else if (!strcmp(s, "STATSERV_HOST")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(StatServ.host, s, sizeof(StatServ.host));
		} else if (!strcmp(s, "STATSERV_USER")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(StatServ.user, s, sizeof(StatServ.user));
		} else if (!strncmp(s, "ENABLE_SPAM", 11)) {
			me.enable_spam = 1;
		} else if (!strncmp(s, "ONLYOPERS", 9)) {
			me.onlyopers = 1;
		} else if (!strncmp(s, "ENABLE_JUPE", 11)) {
			me.enable_jupe = 1;
		} else if (!strncmp(s, "ENABLE_QUOTE", 12)) {
			me.enable_quote = 1;
		} else if (!strcmp(s, "STATSERV_NICK")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(s_StatServ, s, MAXNICK);
		} else if (!strncmp(s, "WANT_PRIVMSG", 12)) {
			me.want_privmsg = 1;
		} else if (!strcmp(s, "SERVICES_NAME")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.services_name, s, MAXHOST);
		} else if (!strcmp(s, "SEND_EXTREME_LAG_NOTICES_TO")) {
			me.send_extreme_lag_notices = 1;
		} else if (!strncmp(s, "ENABLE_HTML", 11)) {
			me.enable_html = 1;
		} else if (!strncmp(s, "SET_OPER", 8)) {
			me.set_oper = 1;
		} else if (!strncmp(s, "SET_DEAF", 8)) {
			me.set_deaf = 1;
		} else if (!strncmp(s, "HIDE_SERVICES", 13)) {
			me.hide_services = 1;
		} else if (!strcmp(s, "HTML_DUMP")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.html, s, sizeof(me.html));
		} else if (!strcmp(s, "HTML_TIME")) {
			int j;
			s = strtok(NULL, "\n");
			strip(s);
			j = atoi(s);
			me.html_time = j;
		} else if (!strcmp(s, "HTML_LINK")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.html_link, s, sizeof(me.html_link));
		} else if (!strcmp(s, "IRCD_TYPE")) {
			printf("The IRCD_TYPE Option has been disabled.\n");
			printf("Please remove the IRCD_TYPE from %s\n", 
				CONFIG_NAME);
			printf("and choose the IRCD_type define in config.h\n");
			printf("\nStats terminated.\n");
			exit(1);
		} else if (!strcmp(s, "CHANNELS")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.join_chans, s, sizeof(me.join_chans));
		} else if (!strcmp(s, "URL")) {
			s = strtok(NULL, "\n");
			strip(s);
			memcpy(me.url, s, sizeof(me.url));
		}
	}
	fclose(fp);
}

void rehash()
{
	/* still nothing, yet */

	/* I need to add in a limited conf reloading - mark flags
	** for what has been changed, send notices etc.  Some thing
	** won't be able to dynamically change, either do a disconnect
	** and reconnect or do a restart when needed... or demand the 
	** admin do it...bloody complicated
	*/

	/* Add an option to reload the user file as well - maybe in
	** a seperate function
	*/

	/* Free and reload the TLD file too */

	/* I should probably have dynamic settings too, and will need
	** a file to store dynamic settings in, read after the main 
	** conf file... gets confusing...bah
	*/
}
