/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
*  Main parsing routine and ircd related commands
*
** GeoStats CVS Identification
** $Id: ircd.c,v 1.41 2000/11/12 23:26:32 wcampbel Exp $
*/

#include "stats.h"
extern const char version_date[], version_time[];

static void ShowMOTD(char *);

void parse(char *line)
{
	char *origin, *cmd, *coreLine;
	int cmdptr = 0;

	strip(line);

	if (!(*line))
		return;

	/* Save a copy of the original line, so we don't trash it
	** when we call strtok or do other manipulations
	** The coreLine is passed into the statserv and spam functions
	** if it's a PRIVMSG
	*/
	coreLine = strdup(line);

#ifdef DEBUG
	log("R: %s", line);
#endif
	/* This strtok parsing really should be replaced with strsep
	** and an argc/argv style set of arguments.  It's much cleaner
	** but portability will be an issue.  I may have to implement 
	** an strsep or a vector library in order to do it properly
	** on machines without strsep
	*/
	origin = strtok(line, " ");
	/* Is the message coming from a server or a user, or is it just
	** a plain text command without an origin
	*/
	if (*origin == ':')
		cmdptr = 1;
	cmd = strtok(NULL, " ");

	if (cmdptr) {
		origin++;
		if (!strcmp(cmd, "VERSION")) {
			sts(":%s %d %s %s :%s -> %s %s", me.name, RPL_VERSION,
				origin, version, me.name, version_date, 
				version_time);
		} else if (!strcmp(cmd, "MOTD")) {
			ShowMOTD(origin);
		} else if (!strcmp(cmd, "PRIVMSG")) {
			char * p;
			/* Grab the target */
			cmd = strtok(NULL, " ");
			if (!strcasecmp(cmd, s_StatServ))
				statserv(coreLine);
			p = strtok(NULL, " ");
			p++;
			/* Can't support &channels
			** NOTE: !channels are HORRIBLE and I have no
			**       desire to support them at this time
			**
			** This is for channel CTCP's
			*/
			if (((cmd[0] == '#') || (cmd[0] == '+')) &&
                            (p[0] == '\001'))
				statserv(coreLine);
			else if (!strcasecmp(cmd, s_Spam) && me.enable_spam)
				spam(coreLine);
#ifdef SUPPORT_MSG_UH
			/* Compare our user@host with what was sent in...
			** If the user is smart enough to send a message to
			** the spam service this way...then he/she is almost
			** unstoppable.  Currently we do not check that.
			*/
                        if (!strcasecmp(cmd, s_UH))
                                statserv(coreLine);
#endif
			/* The only way we can get here...
			** is if we used QUOTE to create a user...
			** Ignore messages to that user
			*/
		} else if (!strcmp(cmd, "SERVER")) {
			char *desc;
			int hops;
#ifdef IRCD_2_10
			int token;
#endif
			cmd = strtok(NULL, " "); 
			desc = strtok(NULL, " ");
			hops = atoi(desc);
#ifdef IRCD_2_10
			/* We have the token here...
			** It appears that the token is NOT global
			** NICK messages use the token to determine 
			** which server owns that user
			**
			** The token is ONLY sent in SERVER with an origin!
			*/
			desc = strtok(NULL, " ");
			token = atoi(desc);
#endif
			desc = strtok(NULL, "\n");
			desc++;
#ifdef IRCD_2_10
			AddServer(cmd, hops, token, desc, origin);
#else
			AddServer(cmd, hops, desc, origin);
#endif
		} else if (!strcmp(cmd, "SQUIT")) {
			Server *s;
			cmd = strtok(NULL, " ");
			/* remember that a user can SQUIT too */
			if ((s = findserver(cmd))) s->stats->numsplits++;
			DelServer(cmd);
		} else if (!strcmp(cmd, "QUIT")) {
			DelUser(origin);
		} else if (!strcmp(cmd, "MODE")) {
			cmd = strtok(NULL, " ");
			if (!strchr(cmd, '#')) {
				cmd = strtok(NULL, "");
				cmd++;
				UserMode(origin, cmd);
			}
#if defined(IRCD_DAL) || defined(IRCD_MLN)
                } else if (!strcmp(cmd, "SVSKILL")) {
			/* Note that we can easily get into a loop
			** if services automatically KILL's our service
			*/
                        cmd = strtok(NULL, " ");
			DelUser(cmd);
                        if (!strcasecmp(cmd, s_StatServ)) {
                                init_statserv();
				init_join();
                        } else if (!strcasecmp(cmd, s_Spam) && 
				 me.enable_spam)
                                init_spam();
                } else if (!strcmp(origin, "SVSNICK")) {
                        char *nnick;
                        User *u;

                        nnick = strtok(NULL, " ");
                        u = finduser(nnick);
                        nnick = strtok(NULL, " ");
                        Change_User(u, nnick);
			/* Catch it if it's one of our clients */
			if (!strcasecmp(cmd, s_StatServ))
				strcpy(s_StatServ, nnick);
			else if (!strcasecmp(cmd, s_Spam) &&
				 me.enable_spam)
				strcpy(s_Spam, nnick);
#endif
		} else if (!strcmp(cmd, "KILL")) {
			User *u = finduser(origin);
			Server *s;
			if (u) {
				u->server->stats->operkills++;
			} else {
				s = findserver(origin);
				if (s)
				s->stats->serverkills++;
			}
			cmd = strtok(NULL, " ");
			DelUser(cmd);
			if (!strcasecmp(cmd, s_StatServ)) {
				init_statserv();
				init_join();
			} else if (!strcasecmp(cmd, s_Spam))
				init_spam();
		} else if (!strcmp(cmd, "PONG")) {
			Server *s;
			cmd = strtok(NULL, " ");
			s = findserver(cmd);
			if (s) {
				s->ping = time(NULL) - ping.last_sent;
				if (ping.ulag > 1)
					s->ping -= (float) ping.ulag;
				if (!strcmp(me.s->name, s->name))
					ping.ulag = me.s->ping;
				if (s->ping > s->highest_ping) {
					s->highest_ping = s->ping;
					s->t_highest_ping = time(NULL);
				} else if (s->ping < s->lowest_ping) {
					s->lowest_ping = s->ping;
					s->t_lowest_ping = time(NULL);
				}
			} else {
				log("Received PONG from unknown server: %s", 
					cmd);
			}
#if defined(IRCD_DAL) || defined(IRCD_MLN)
		} else if (!strcmp(cmd, "AWAY")) {
			/* Not used on Hybrid */
			User *u = finduser(origin);
			if (u->is_away) {
				me.away--;
				u->is_away = 0;
			} else {
				me.away++;
				u->is_away = 1;
			}
#endif
		} else if (!strcmp(cmd, "NICK")) {
			User *u = finduser(origin);
			if (u) {
				cmd = strtok(NULL, " ");
#ifdef IRCD_2_10
				/* I don't know why IRCnet puts the : here
				** but then leaves it out on other commands
				** (ie STATS) that other IRCD's use
				*/
				cmd++;
#endif
				Change_User(u, cmd);
			}
		} else if (!strcmp(cmd, "PING")) {
			char* p = strtok(NULL, " ");
			char* q = strtok(NULL, " ");
			if ((p && q)) {
				q++;
				sts(":%s PONG %s :%s", me.name, q, p);
			}
		} else if (!strcmp(cmd, "STATS")) {
			char* type = strtok(NULL, " ");
			char* target = strtok(NULL, " ");
			User *u = finduser(origin);
			if ((target && type)) {
#ifndef IRCD_2_10
				/* Everything except IRCnet puts a : here
				** WHY?!?!
				*/
				target++;
#endif
				if (!strcasecmp(me.name, target))
				    if (u)
					ParseStats(u, type[0]);
			}
		} else if (!strcmp(cmd, "TRACE")) {
			User *u = finduser(origin);
			if (u)
				Trace(u);
		}
	} else {
		if (!strcmp(origin, "PING")) {
			sts("PONG %s", cmd);
		} else if (!strcmp(origin, "PASS")) {
			init_statserv();
			init_spam();
			init_join();
		} else if (!strcmp(origin, "SERVER")) {
			char *desc;
			int hops;
			Server *s;
			desc = strtok(NULL, " ");
			hops = atoi(desc);
			desc = strtok(NULL, "\n");
			desc++;
			/* The hops is passed into us... It will always
			** be 1 for the uplink (the only way we will get 
			** a SERVER without an origin), but if it's given
			** to us, we might as well use it.
			*/
#ifdef IRCD_2_10
			/* From my testing, it appears as the uplink
			** always has a token of 1
			*/
			AddServer(cmd, hops, 1, desc, me.name);
#else
			AddServer(cmd, hops, desc, me.name);
#endif
			s = findserver(cmd);
			me.s = s;
		} else if (!strcmp(origin, "SQUIT")) {
			Server *s = findserver(cmd);
			if (s) s->stats->numsplits++;
			DelServer(cmd);
		} else if (!strcmp(origin, "NICK")) {
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6)
			char *user, *host, *server, *modes; 
			user = strtok(NULL, " ");
			user = strtok(NULL, " ");
			modes = strtok(NULL, " "); 
			user = strtok(NULL, " ");
			host = strtok(NULL, " ");
			server = strtok(NULL, " ");
			AddUser(cmd, user, host, server);
			UserMode(cmd, modes); 
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN)
			char *user, *host, *server;
			user = strtok(NULL, " ");
			user = strtok(NULL, " ");
			user = strtok(NULL, " ");
			host = strtok(NULL, " ");
			server = strtok(NULL, " ");
			AddUser(cmd, user, host, server);
#else
#if defined(IRCD_2_10)
			/* Here is where we actually use the server token */
			char *user, *host, *modes;
			int token;
			Server *s;
			user = strtok(NULL, " ");
			user = strtok(NULL, " ");
			host = strtok(NULL, " ");
			modes = strtok(NULL, " ");
			token = atoi(modes);
			s = findservtoken(token);
			modes = strtok(NULL, " ");
			/* ugh...this is currently a performance hit too, as
			** I must go through the server table twice, once 
			** with the token, once with the name.  This IS on
			** the TODO list and will be fixed shortly 
			*/
			AddUser(cmd, user, host, s);
			UserMode(cmd, modes);
#endif
#endif
#endif
#if defined(IRCD_MLN) || defined(IRCD_DAL)
		/* SVSNICK and SVSKILL are only in dal based servers */
                } else if (!strcmp(cmd, "SVSKILL")) {
                        /* Note that we can easily get into a loop
                        ** if services automatically KILL's our service
                        */
                        cmd = strtok(NULL, " ");
                        DelUser(cmd);
                        if (!strcasecmp(cmd, s_StatServ)) {
                                init_statserv();
				init_join();
                        } else if (!strcasecmp(cmd, s_Spam) &&
				 me.enable_spam)
                                init_spam();
                } else if (!strcmp(origin, "SVSNICK")) {
			/* Should this be placed up above with an 
			** origin?  This should also do special 
			** checks to see if one of our nicks is
			** forcibly changed by services
			*/
                        char *nnick;
                        User *u;
       
                        nnick = strtok(NULL, " ");
                        u = finduser(nnick);
                        nnick = strtok(NULL, " ");
                        Change_User(u, nnick);
                        if (!strcasecmp(cmd, s_StatServ))
                                strcpy(s_StatServ, nnick);
                        else if (!strcasecmp(cmd, s_Spam) &&
                                 me.enable_spam)
                                strcpy(s_Spam, nnick);
#endif
                }

	}
	free(coreLine);
}

/* Send a message to a user from a user */
void privmsg(char *to, const char *from, char *fmt, ...)
{
	va_list ap;
	char buf[512], buf2[512];

	va_start(ap, fmt);
	vsnprintf(buf2, sizeof(buf2), fmt, ap);
	if (me.want_privmsg)
		sprintf(buf, ":%s PRIVMSG %s :%s", from, to, buf2);
	else
		sprintf(buf, ":%s NOTICE %s :%s", from, to, buf2);
	sts("%s", buf);
	va_end(ap);
}

/* This sends an array of character arrays in a PRIVMSG or NOTICE
** to a user from a user created by the service (typically s_StatServ
** but it's generic enough to be used for future extensions
*/
void privmsg_list(char *to, char *from, const char **text)
{
	while (*text) {
		if (**text)
			privmsg(to, from, "%s", *text);
		else
			privmsg(to, from, " ");
		text++;
	}	
}

/* Do GLOBOPS on Dal based servers, and WALLOPS on hybrid based and IRCnet 
** servers 
*/
void globops(char *from, char *fmt, ...)
{
	va_list ap;
	char buf[512], buf2[512];

	va_start(ap, fmt);
	vsnprintf(buf2, sizeof(buf2), fmt, ap);
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_2_10)
		sprintf(buf, ":%s WALLOPS :%s", from, buf2);
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN)
		sprintf(buf, ":%s GLOBOPS :%s", from, buf2);
#endif
#endif
	sts("%s", buf);
	va_end(ap);
}

/* This function KILL's or optionally ignores a user for flooding services. */
int flood(User *u)
{
	/* Our "opers" are immune to flood control */
	if (u->myuser) 
		return 0;

	/* Flood control is disabled by setting the flood lines to 0 */
	if (me.flood) {
		time_t current = time(NULL);

		if (current - u->t_flood > me.flood_time) {
			u->t_flood = time(NULL);
			u->flood = 0;
			u->flood_sent = 0;
			return 0;
		}

		if (u->flood >= me.flood) {
		   /* This was a switch in the initial writing of this
		   ** For now (unless I see a specific reason against
		   ** using switch, it can stay a switch)
		   */
		   switch (me.flood_action) {
		     case 0:
			sts(":%s KILL %s :%s!%s (Flooding Services.)",
				s_StatServ, u->nick, StatServ.host, 
				s_StatServ);
			log("FLOODING (KILLED): %s!%s@%s", u->nick, 
				u->username, u->hostname);
			/* This is an unnecessary piece of overhead ugh */
			/* We KILL the user, we should remove them from 
			** the databases...
			*/
			DelUser(u->nick);
			break;
		     case 1:
			/* Only send the flood WALLOPS/GLOBOPS message
			** once, or WE will be the ones flooding
			*/
			if (!u->flood_sent) {
			  globops("%s is Flooding Services", u->nick);
			  log("FLOODING (IGNORE): %s!%s@%s", u->nick,
				  u->username, u->hostname);
			}
			break;
		   }
		   return 1;
		} else {
			u->flood++;
		}
	}
	return 0;
}

/* This is left in unmodified as to the original author's request */
static void ShowMOTD(char *nick)
{
	static unsigned char copyright[38] = {  /* Can you tell I was bored? */
	0x47, 0x65, 0x6f, 0x53, 0x74, 0x61, 0x74, 0x73, 0x20, 0x61, 0x72, 0x65,
	0x20, 0x28, 0x63, 0x29, 0x20, 0x31, 0x39, 0x39, 0x39, 0x20, 0x4a, 0x6f,
	0x6e, 0x61, 0x74, 0x68, 0x61, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
	0x65, 0x0
	};

	/* Removal or modification of this function is against the law.
	** Be nice, and don't remove it or modify it.
	** -Jonathan 'netgod' George (net@lite.net) */

	sts(":%s 375 %s :- %s Message of the Day", me.name, nick, me.name);
	sts(":%s 372 %s :- ", me.name, nick);
	sts(":%s 372 %s :- %s", me.name, nick, copyright);
	sts(":%s 372 %s :- Grab your copy at ftp.lite.net", me.name, nick);
	sts(":%s 376 %s :End of /MOTD command.", me.name, nick);
}

/* Send out QUIT messages as a courtesy... The only one who really 
** matters is s_StatServ, as s_Spam will never be in a channel...
** unless the Spam service is rewritten to join channels looking for 
** ON JOIN spam... Then we get into some heavy privacy issues :/
*/
void SendQuit(User *u, int restart, char * comm)
{
	sts(":%s QUIT :%s Request by %s: %s", s_StatServ, 
	    (restart) ? "Restart" : "Shutdown", u->nick,
            (comm) ? comm : "No Reason");
	if (me.enable_spam)
		sts(":%s QUIT :Shutdown Request by %s: %s", s_Spam, 
		(restart) ? "Restart" : "Shutdown", u->nick,
                (comm) ? comm : "No Reason");
}

/* This is how we answer STATS requests...
** This is very Hybrid-like at the moment.  It should not affect other
** ircd's.  If someone can refine this for other ircd's, I will patch it in
*/
void ParseStats(User *u, char type)
{
    MyUser *myu = NULL;
    int count = 0;

    switch(type) {
	case 'c':
		/* Our only "C and N" lines are to our uplink, show the
		** real IP/address of our uplink to opers, but use the
		** Hybrid6 method of IP hiding for non-opers.  This is
		** moot if your ircd blocks remote STATS from non-opers
		*/
		sts(":%s %d %s C %s * %s %d 2", me.name, RPL_STATSCLINE, 
			u->nick, (u->is_oper) ? me.uplink : "*@127.0.0.1", 
			me.s->name, me.port);
		sts(":%s %d %s N %s * %s 0 2", me.name, RPL_STATSNLINE, 
			u->nick, (u->is_oper) ? me.uplink : "*@127.0.0.1", 
			me.s->name);
		sts(":%s %d %s c :End of STATS report", me.name, 
			RPL_ENDOFSTATS, u->nick);
		break;
	case 'i':
		/* This is purely cosmetic, show a closed server */
		sts(":%s %d %s I * * *@%s 0 1", me.name, RPL_STATSILINE, 
			u->nick, me.name);
                sts(":%s %d %s i :End of STATS report", me.name,
                        RPL_ENDOFSTATS, u->nick);
		break;
	case 'h':
		/* Our only hub should be our uplink.  This is cosmetic
		** as well.
		*/
		sts(":%s %d %s H * * %s 0 -1", me.name, RPL_STATSHLINE,
			u->nick, me.s->name);
                sts(":%s %d %s h :End of STATS report", me.name,
                        RPL_ENDOFSTATS, u->nick);
		break;
	case 'o':
	    /* This STATS is actually very useful.  This shows a list of
	    ** our users (all who can use LOGIN, the equivalent of opers)
	    */
	    for (myu = myuhead; myu; myu = myu->next) {
		sts(":%s %d %s O *@* * %s 0 10", me.name, RPL_STATSOLINE,
			u->nick, myu->username);
	    }
            sts(":%s %d %s o :End of STATS report", me.name,
                    RPL_ENDOFSTATS, u->nick);
            break;
	case 'p':
	    /* This STATS is also relatively useful.  It does the Hybrid
	    ** STATS p - show all online opers.  All users who have used
	    ** LOGIN will be shown as Stats opers.
	    ** NOTE:  We now DO print out the current user's nick.
	    */
	    for (myu = myuhead; myu; myu = myu->next) {
		if (myu->ison) {
		    sts(":%s %d %s :[O] %s", me.name, RPL_STATSDEBUG,
			u->nick, myu->u->nick);
		    count++;
		}
	    }
	    sts(":%s %d %s :%d OPER%s", me.name, RPL_STATSDEBUG, u->nick,
		count, (count == 1) ? "" : "s");
	    sts(":%s %d %s p :End of STATS report", me.name,
		RPL_ENDOFSTATS, u->nick);
	    break;
	case 'u':
		/* Send our uptime in the usual fasion */
		sts(":%s %d %s :%s", me.name, RPL_STATSUPTIME,
			u->nick, uptime(me.t_start));
		sts(":%s %d %s u :End of STATS report", me.name,
			RPL_ENDOFSTATS, u->nick);
		break;
	case 'y':
		/* Purely cosmetic with fake values...EXCEPT for the 
		** reconnect time in the server Y line (2)
		*/
		sts(":%s %d %s Y 1 0 0 2 000000000", me.name, RPL_STATSYLINE,
			u->nick);
		sts(":%s %d %s Y 2 0 %d 1 000000000", me.name, 
			RPL_STATSYLINE, u->nick, me.r_time);
		sts(":%s %d %s Y 10 0 0 1 000000000", me.name, 
			RPL_STATSYLINE, u->nick);
		sts(":%s %d %s y :End of STATS report", me.name,
                        RPL_ENDOFSTATS, u->nick);
		break;
	default:
		sts(":%s %d %s * :End of STATS report", me.name,
                        RPL_ENDOFSTATS, u->nick);
                break;
    }
}

/* This functionality is unfinished....I also don't currently know 
** what I will report to non-opers.  Dal/mln TRACE replies are untested 
** as well, and we don't report JUPEs
** There is also NO IRCnet 2.10.x support yet
*/
void Trace(User *u)
{
    if (u->is_oper) {
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6)
	   sts(":%s %d %s %s %d %s[%s@%s] (127.0.0.1) 000000000 000000000",
		me.name, (me.set_oper) ? RPL_TRACEOPERATOR : RPL_TRACEUSER, 
		u->nick, 
                (me.set_oper) ? "Oper" : "User", (me.set_oper) ? 10 : 1,
		s_StatServ, StatServ.user, StatServ.host);
           if (me.enable_spam)
		sts(":%s %d %s User 1 %s[please@%s] (127.0.0.1) 000000000 000000000",
		    me.name, RPL_TRACEUSER, u->nick, s_Spam, me.name);
           sts(":%s %d %s Serv 2 %dS %ldC %s AutoConn.!*@%s 000000000",
                    me.name, RPL_TRACESERVER, u->nick, me.maxservers, 
		    me.maxusers, me.s->name, me.name);
	   sts(":%s %d %s %s :End of TRACE", me.name, RPL_ENDOFTRACE, u->nick, 
		    me.name);
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN)
	   sts(":%s %d %s %s %d %s[%s] 0", me.name, 
               (me.set_oper) ? RPL_TRACEOPERATOR : RPL_TRACEUSER, u->nick, 
	       (me.set_oper) ? "Operator" : "User",
	       (me.set_oper) ? 10 : 1, s_StatServ, StatServ.host);
	   if (me.enable_spam)
		sts(":%s %d %s User 1 %s[%s] 0", me.name, RPL_TRACEUSER, 
                    u->nick, s_Spam, me.name);
	   sts(":%s %d %s Server 2 %dS %ldC %s AutoConn.!*@%s 0",
		me.name, RPL_TRACESERVER, u->nick, me.maxservers, me.maxusers, 
		me.s->name, me.name);
#endif
#endif
    }
}
