/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: main.c,v 1.24 2000/11/12 23:26:32 wcampbel Exp $
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
** See the file called 'LICENSE' for more details.
*/

#include "stats.h"

char s_StatServ[MAXNICK] = "StatServ";
char s_Spam[8] = "cm-i";
int servsock;
const char version[] = "Stats $Revision: 1.24 $";
const char version_date[] = __DATE__;
const char version_time[] = __TIME__;

#ifdef SUPPORT_MSG_UH
char s_UH[MAXUSER + MAXHOST + 2];
#endif

/* NOTE:  This code needs profiled to find out what needs to be optimized
** to make the code as efficient as possible.
*/
static void start();

int main()
{
	int forked = 0;
	FILE *fp;

	log("Statistics Started (%s).", version);
	me.t_start = time(NULL);

	InitDefaults();
	ConfLoad();

	/* Sanity checking for FLOOD */
	if (me.flood == 1) {
		me.flood = 2;
	}

#ifdef SUPPORT_MSG_UH
	/* Create the user@host that we are to save some cycles later...
	** IF this features is used often
	*/
	sprintf(s_UH, "%s@%s", StatServ.user, StatServ.host);
#endif

	/* This should ONLY be run once */
	srandom(time(NULL)); 

	LoadStats();
	LoadTLD();
	LoadMyUsers();

#ifndef DEBUG
	forked=fork();
#endif
	if (forked) {
		fp = fopen(PIDFILE, "w");
		fprintf(fp, "%i", forked);
		fclose(fp);
		return 0;
	}

	start();

	return 1;
}

void start()
{
	TimerReset();
	me.users = me.opers = me.servers = 0;
	init_server_hash();
	init_user_hash();
	init_tld();

	log("Connecting to %s:%d", me.uplink, me.port);
	if (servsock > 0)
		close(servsock);

	servsock = ConnectTo(me.uplink, me.port);

	if (servsock <= 0) {
		log("Unable to connect to %s", me.uplink);
	} else {
		login();
		read_loop();
	}
	log("Reconnecting to the server in %d seconds", me.r_time);
	sleep(me.r_time);
	start();
}

/* This is ugly, logging into each server type is slightly different */
void login()
{
#if defined(IRCD_HYBRID)
	sts("PASS %s :TS", me.pass);
	sts("SERVER %s 1 :%s", me.name, me.desc);
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN)
	sts("PASS %s", me.pass);
	sts("SERVER %s 1 :%s", me.name, me.desc);
#else
#if defined(IRCD_HYB6)
	sts("PASS %s :TS", me.pass);
	sts("CAPAB :QS");
	sts("SERVER %s 1 :%s", me.name, me.desc);
#else
#if defined(IRCD_2_10)
	/* I currently don't send the "version flags" or that P strict crap */
	sts("PASS %s :0210", me.pass);
	sts("SERVER %s 1 :%s", me.name, me.desc);
#endif
#endif
#endif
#endif
}

/* Init the StatServ client */
void init_statserv()
{
	char mbuf[MAXUSER];

	sprintf(mbuf, "+%s%s", (me.set_oper) ? "o" : "", 
		(me.set_deaf) ? "d" : "");

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6)
	sts("NICK %s 1 1 %s %s %s %s :/msg %s HELP", s_StatServ,
		mbuf, StatServ.user, StatServ.host, me.name, s_StatServ);
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN)
	sts("NICK %s 1 1 %s %s %s 0 :/msg %s HELP", s_StatServ,
		StatServ.user, StatServ.host, me.name, s_StatServ);
	/* If it's anything EXCEPT just a plain "+", send the mode line */
	if (strcmp(mbuf, "+")) 
		sts(":%s MODE %s %s", s_StatServ, s_StatServ, mbuf);
#else
#if defined(IRCD_2_10)
	sts("NICK %s 1 %s %s %d %s :/msg %s HELP", s_StatServ,
		StatServ.user, StatServ.host, 
		/* This is the server token (which is ugly as sin) 
		** For now, I'll send '1'... Once I find out more about
		** it, I'll send the proper value.  
		*/
		1, mbuf, s_StatServ);
#endif
#endif
#endif

#ifdef IRCD_2_10
	AddUser(s_StatServ, StatServ.user, StatServ.host, findserver(me.name));
#else
	AddUser(s_StatServ, StatServ.user, StatServ.host, me.name);
#endif
}

/* Bring up the Spam detecting client if it was enabled */
void init_spam()
{
	if (!me.enable_spam)
		return;

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6)
	sts("NICK %s 1 1 + please %s %s :Spam Me!", s_Spam,
		me.name, me.name);
#else
#if defined(IRCD_DAL) || defined(IRCD_MLN)
	sts("NICK %s 1 1 please %s %s :Spam Me!", s_Spam,
		me.name, me.name);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 please %s %d + :Spam Me!", s_Spam,
                me.name, 1);
                /* This is the server token (which is ugly as sin)
                ** For now, I'll send '1'... Once I find out more about
                ** it, I'll send the proper value.
                */
#endif
#endif
#endif

#ifdef IRCD_2_10
	AddUser(s_Spam, "please", me.name, findserver(me.name));
#else
	AddUser(s_Spam, "please", me.name, me.name);
#endif
}

/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Stats
*/
void init_join()
{
   char *c;
   if (me.join_chans) {
	for (c = strtok(me.join_chans, ",");
	     c;
	     c = strtok(NULL, ",")) {
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6)
			sts(":%s SJOIN %ld %s + :@%s", me.name, 
			    time(NULL), c, s_StatServ);
#else
#if defined(IRCD_DAL)
			sts(":%s JOIN %s", s_StatServ, c);
#else
#if defined(IRCD_MLN)
			sts(":%s JOIN %s", s_StatServ, c);
			sts(":%s MODE %s +o %s %ld", me.name, c,
			s_StatServ, time(NULL));
#else
#if defined(IRCD_2_10)
			/* Here is the only one that we hack ops... */
			sts(":%s NJOIN %s :@%s", me.name, c, s_StatServ);
#endif
#endif
#endif
#endif
	}
   }
}

/* our own malloc with logging and sanity checking */
void *smalloc(long size)
{
	void *buf;

	if (!size) {
		log("smalloc(): illegal attempt to allocate 0 bytes!");
		size = 1;
	}
	buf = malloc(size);
	if (!buf) {
		log("smalloc(): out of memory.");
		exit(0);
	}
	return buf;
}

/* our own strdup with logging and sanity checking */
char *sstrdup(const char *s)
{
	char *t = strdup(s);
	if (!t) {
		log("sstrdup(): out of memory.");
		exit(0);
	}
	return t;
}

/* Generate a hash value into the tables */
unsigned long HASH(const unsigned char *name, int size_of_table)
{
	unsigned long h = 0, g;

	while (*name) {
		h = (h << 4) + *name++;
			if ((g = h & 0xF0000000))
				h ^= g >> 24;
			h &= ~g;
	}
	return h % size_of_table;
}

/* Lowercase a string */
char *strlower(char *s)
{
	char *t = s;
	while ((*t++ = tolower(*t)))
		;
	return s;
}
