/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: sock.c,v 1.12 2000/11/12 23:13:43 wcampbel Exp $
*/

#include "stats.h"

fd_set readfds, nullfds;

/* Initiate the connection to the server, returns the FD */
int ConnectTo(char *host, int port)
{
	struct hostent *hp;
	struct sockaddr_in sa;
	struct in_addr * in;
	int s;

	if ((hp = gethostbyname (host)) == NULL) {
		return (-1);
	}

	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		return (-1);

        if (me.use_vhost) {
                bzero(&sa, sizeof(sa));
                sa.sin_family = AF_INET;
                if ((hp = gethostbyname (me.vhost)) == NULL) {
                        return (-1);
                }
                in = (struct in_addr *) (hp->h_addr_list[0]);
                sa.sin_addr.s_addr = in->s_addr;
                sa.sin_port = 0;
                if (bind(s, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
                        close(s);
			printf("Unable to bind to VHOST %s", me.vhost);
                        return (-1);
                }
        }

	bzero((void *) &sa, sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_port = htons (port);
    bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);

	if (connect (s, (struct sockaddr *) &sa, sizeof (sa)) < 0) {
		close (s);
		return (-1);
	}

	return s;
}

/* Here is where it all happens, get a line from the server
** and send it off to the parse function.  Also this is where
** chk, the timer function, is called.  
*/
void read_loop()
{
	register int i, j, SelectResult;
	struct timeval TimeOut;
	char c;
	char buf[BUFSIZE];

	while (1) {
		memset(buf, '\0', BUFSIZE);
		chk();
		FD_ZERO(&readfds);
		TimeOut.tv_sec = 500;
		TimeOut.tv_usec = 0L;
		FD_SET(servsock, &readfds);
	if ((SelectResult = select(FD_SETSIZE, &readfds, &nullfds, &nullfds,
			&TimeOut)) > 0) {
			for (j = 0; j < BUFSIZE; j++) {
				if (FD_ISSET(servsock, &readfds)) {
					i = read(servsock, &c, 1);
					if (i !=0) {
						buf[j] = c;
						if ((c == '\n') || (c == '\r')) {
							parse(buf);
							break;
						}
					} else
						return;
				} else
					break;
			}
		} else {
			if (SelectResult == 0) {
				log("Lost connection to server.");
				return;
			}
		}
	}
}

/* send a line of text to the server, Append the \n */
void sts(char *fmt,...)
{
	va_list ap;
	char buf[512];

	va_start (ap, fmt);
	vsnprintf (buf, 512, fmt, ap);

#ifdef DEBUG
	log("SENT: %s", buf);
#endif
	strcat (buf, "\n");

	if ((write (servsock, buf, strlen (buf))) == -1) {
		log("Write error.");
		exit(0);
	}

	va_end (ap);
}

/* Generic logging routine */
void log(char *fmt, ...)
{
	va_list ap;
	FILE *logfile;
	char buf[512], fmtime[80];
	time_t tmp = time(NULL);

	va_start(ap, fmt);
	vsnprintf(buf, 512, fmt, ap);

	strftime(fmtime, 80, TIMEFMT, localtime(&tmp));

	if ((logfile = fopen(LOGFILE, "a")) == NULL) return;

	if (logfile)
		fprintf(logfile, "(%s) %s\n", fmtime, buf);

#ifdef DEBUG
	fprintf(stderr, "(%s) %s\n", fmtime, buf);
#endif

	fclose(logfile);
	va_end(ap);
}

/* Rotate the logfile, called from chk() */
void ResetLogs()
{
	char tmp[25];
	time_t t = time(NULL);

	strftime(tmp, 25, BACKUPLOGFMT, localtime(&t));
	rename(LOGFILE, tmp);
	log("Started fresh logfile.");
}

/* Our own ctime function */
char *sctime(time_t stuff)
{
	char *s, *c;

	s = ctime(&stuff);
	if ((c = strchr(s, '\n'))) *c = '\0';

	return s;
}

char fmtime[80];

/* custom ftime format */
char *sftime(time_t stuff)
{
	struct tm *ltm = localtime(&stuff);

	strftime(fmtime, 80, "%A, %d %b %Y, %I:%M%p %Z", ltm);

	return fmtime;
}

#ifdef MD5
/* Create a random MD5 based salt for password
** encryption
*/
char* make_salt()
{
	static char saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
	char * salt = malloc(4);
	char * retSalt = malloc(8);

	salt[0] = saltChars[random() % 64];
	salt[1] = saltChars[random() % 64];
	salt[2] = saltChars[random() % 64];
	salt[3] = 0;

	sprintf(retSalt, "$1$%s$", salt);
	free(salt);

	return retSalt;
}
#endif

#ifdef CRYPT
/* create a random salt for password encryption 
** This only creates DES based salts...
**
** NOTE:  This code is partially derived from mkpasswd.c
**        by Nelson Minar (minar@reed.edu)
*/
char* make_salt()
{
	static char saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
	char * salt = malloc(3);

	salt[0] = saltChars[random() % 64];
	salt[1] = saltChars[random() % 64];
	salt[2] = 0;

	return salt;
}
#endif /* CRYPT */
