/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: stats.c,v 1.4 2000/11/10 20:49:00 wcampbel Exp $
*/

#include "stats.h"

Stats *shead;
TLD *tldhead;
static Stats *new_stats(const char *);

static char buf[100];

/* uptime displayed as a string */
char *uptime(time_t when)
{
	time_t u = time(NULL) - when;

	if (u > 86400)
		sprintf(buf, "Statistics up %ld day%s, %02ld:%02ld",
			u/86400, (u/86400 == 1) ? "" : "s",
			(u/3600) % 24, (u/60) % 60);
	else if (u > 3600)
		sprintf(buf, "Statistics up %ld hour%s, %ld minute%s",
			u/3600, u/3600==1 ? "" : "s",
			(u/60) % 60, (u/60)%60 == 1 ? "" : "s");
	else
		sprintf(buf, "Statistics up %ld minute%s, %ld second%s",
			u/60, u/60 == 1 ? "" : "s",
			u%60, u%60 == 1 ? "" : "s");

	return buf;
}

/* "net" -- not DOT"net" */
TLD *findtld(char *tld)
{
	TLD *t;

	for (t = tldhead; t; t = t->next) {
		if (!strcasecmp(t->tld, tld))
			return t;
	}

	return NULL;
}

/* Add the TLD entry to a user record */
TLD *AddTLD(User *u)
{
	TLD *t = NULL;
	char *m;

	m = strrchr(u->hostname, '.');

	if (!m)
		t = findtld("num");
	else
		m++;

	if (!t) {
		if (!isdigit((int) *m))
			t = findtld(m);
		else
			t = findtld("num");
	}

	if (!t) {
		log("Unable to find TLD entry for %s (%s)", u->nick, m);
		return NULL;
	}
	t->users++;

	return t;
}

/* Load the TLD file */
void LoadTLD()
{
	register int i;
	FILE *fp;
	char buf[BUFSIZE], buf2[BUFSIZE];
	char *tmp = NULL, *tmp2 = NULL;
	TLD *t;

	if ((fp = fopen(TLD_FILE, "r")) == NULL) {
		log("Top Level Domain Statistics not loaded: file not found.");
		return;
	}

    while (fgets(buf, BUFSIZE, fp)) {
		memcpy(buf2, buf, sizeof(buf));
		tmp = strrchr(buf, '(');
		tmp++;
		tmp[strlen(tmp) - 1] = '\0';
		tmp[strlen(tmp) - 1] = '\0';
		for (i = 0; buf[i] != '('; i++)
			buf2[strlen(buf2) + 1] = buf[i];

		if ((tmp2 = strchr(buf2, '\n')))	*tmp2 = '\0';

		t = smalloc(sizeof(TLD));
		t->users = 0;
		t->country = sstrdup(buf2);
		memcpy(t->tld, tmp, sizeof(t->tld));

		if (!tldhead) {
			tldhead = t;
			tldhead->next = NULL;
		} else {
			t->next = tldhead;
			tldhead = t;
		}
    }
	fclose(fp);
}

void init_tld()
{
	TLD *t;

	for (t = tldhead; t; t = t->next)
		t->users = 0;
}

void IncreaseOpers(Server *s)
{
#ifdef DEBUG
	log("Increasing OperCount for %s", s->name);
#endif
	s->opers++;
	me.opers++;
	if (s->opers > s->stats->maxopers) {
		s->stats->maxopers = s->opers;
		s->stats->t_maxopers = time(NULL);
	}
	if (me.opers > me.maxopers) {
		me.maxopers = me.opers;
		me.t_maxopers = time(NULL);
	}
	if (me.opers > daily.opers) {
		daily.opers = me.opers;
		daily.t_opers = time(NULL);
	}
}

void IncreaseUsers(Server *s)
{
#ifdef DEBUG
	log("Increasing UserCount for %s", s->name);
#endif
	s->users++;
	me.users++;
	if (s->users > s->stats->maxusers) {
		s->stats->maxusers = s->users;
		s->stats->t_maxusers = time(NULL);
	}
	if (me.users > me.maxusers) {
		me.maxusers = me.users;
		me.t_maxusers = time(NULL);
	}
	if (me.users > daily.users) {
		daily.users = me.users;
		daily.t_users = time(NULL);
	}
}

void IncreaseServers(Server *s)
{
#ifdef DEBUG
	log("Increasing server count");
#endif
	me.servers++;
	if (me.servers > me.maxservers) {
		me.maxservers = me.servers;
		me.t_maxservers = time(NULL);
	}
	if (me.servers > daily.servers) {
		daily.servers = me.servers;
		daily.t_servers = time(NULL);
	}
}

/* Create a new Stats struct for a server we haven't seen before */
static Stats *new_stats(const char *name)
{
	Stats *s = calloc(sizeof(Stats), 1);

#ifdef DEBUG
	log("new_stats(%s)", name);
#endif

	if (!s) {
		log("Out of memory.");
		exit(0);
	}

	memcpy(s->name, name, MAXHOST);
	s->numsplits = 0;
	s->maxusers = 0L;
	s->t_maxusers = time(NULL);
	s->t_maxopers = time(NULL);
	s->maxopers = 0;
	s->lastseen = time(NULL);
	s->starttime = time(NULL);

	if (!shead) {
		shead = s;
		shead->next = NULL;
	} else {
		s->next = shead;
		shead = s;
	}
	return s;
}

void AddStats(Server *s)
{
	Stats *st = findstats(s->name);

#ifdef DEBUG
	log("AddStats(%s)", s->name);
#endif

	if (!st) {
		st = new_stats(s->name);
		s->stats = st;
	} else {
		s->stats = st;
		st->lastseen = time(NULL);
	}
}

Stats *findstats(char *name)
{
	Stats *t;
#ifdef DEBUG
	log("findstats(%s)", name);
#endif
	for (t = shead; t; t = t->next)
		if (!strcasecmp(name, t->name))
			return t;
	return NULL;
}

void SaveStats()
{
	FILE *fp = fopen(STATS_DB, "w");
	Stats *s;

	if (!fp) {
		log("Unable to open %s for writing.", STATS_DB);
		return;
	}

	for (s = shead; s; s = s->next) {
#ifdef DEBUG
	log("Writing statistics to database for %s", s->name);
#endif
		fprintf(fp, "%s %d %ld %ld %d %ld %ld %ld %d %d\n", s->name,
			s->numsplits, s->maxusers, s->t_maxusers, s->maxopers,
			s->t_maxopers, s->lastseen, s->starttime, s->operkills,
			s->serverkills);
	}
	fclose(fp);

	if ((fp = fopen(NSTATS_DB, "w")) == NULL) {
		log("Unable to open %s for writing.", NSTATS_DB);
		return;
	}
	fprintf(fp, "%d %ld %d %ld %ld %ld\n", me.maxopers, me.maxusers,
		me.maxservers, me.t_maxopers, me.t_maxusers, me.t_maxservers);
	fclose(fp);
}

/* NOTE that if the format of the statistics database is incorrect.  Stats
** WILL crash.  I hope you kept backups...
*/
void LoadStats()
{
	FILE *fp = fopen(NSTATS_DB, "r");
	Stats *s;
	char buf[BUFSIZE];
	char *name, *numsplits, *maxusers, *t_maxusers,
		*maxopers, *t_maxopers, *lastseen, *starttime,
		*operkills, *serverkills;

	if (fp) {
	while (fgets(buf, BUFSIZE, fp)) {
		me.maxopers = atoi(strtok(buf, " "));
		me.maxusers = atol(strtok(NULL, " "));
		me.maxservers = atoi(strtok(NULL, " "));
		me.t_maxopers = atoi(strtok(NULL, " "));
		me.t_maxusers = atol(strtok(NULL, " "));
		me.t_maxservers = atoi(strtok(NULL, " "));
	}
	fclose(fp);
	}

	if ((fp = fopen(STATS_DB, "r")) == NULL)
		return;

	memset(buf, '\0', BUFSIZE);
	while (fgets(buf, BUFSIZE, fp)) {
		s = smalloc(sizeof(Stats));
		name = strtok(buf, " ");
		numsplits = strtok(NULL, " ");
		maxusers = strtok(NULL, " ");
		t_maxusers = strtok(NULL, " ");
		maxopers = strtok(NULL, " ");
		t_maxopers = strtok(NULL, " ");
		lastseen = strtok(NULL, " ");
		starttime = strtok(NULL, " ");
		operkills = strtok(NULL, " ");
		serverkills = strtok(NULL, " ");

		memcpy(s->name, name, MAXHOST);
		s->numsplits = atoi(numsplits);
		s->maxusers = atol(maxusers);
		s->t_maxusers = atol(t_maxusers);
		s->maxopers = atoi(maxopers);
		s->t_maxopers = atol(t_maxopers);
		s->lastseen = atol(lastseen);
		s->starttime = atol(starttime);
		s->operkills = atoi(operkills);
		s->serverkills = atoi(serverkills);

#ifdef DEBUG
	log("LoadStats(): Loaded statistics for %s", s->name);
#endif
		if (!shead) {
			shead = s;
			shead->next = NULL;
		} else {
			s->next = shead;
			shead = s;
		}
	}
	fclose(fp);
}
