/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: stats.h,v 1.49 2000/11/12 23:26:33 wcampbel Exp $
*/

#ifndef STATS_H
#define STATS_H

/* Find out which of these are not needed */
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/time.h>
#include <ctype.h>
#include <sys/stat.h>
#include "config.h"

/* Standard settings... ONLY change these if you know what
** you are doing.  The service may fail to work if not
*/
#define BUFSIZE			512
#define CONFIG_NAME		"stats.cfg"
#define TLD_FILE		"data/tlds.nfo"
#define STATS_DB		"data/stats.db"
#define NSTATS_DB		"data/nstats.db"
#define PIDFILE			"stats.pid"
#define LOGFILE			"stats.log"
#define USERFILE		"users.lst"
#define BACKUPLOGFMT		"logs/stats-%m-%d.log"
#define TIMEFMT			"%H:%M[%m/%d/%Y]"
#define MAXHOST			63
#define MAXPASS			32
#define MAXNICK			32
#define MAXUSER			10
#define LINE			80

#ifdef LARGE_NET
# define S_TABLE_SIZE 150
# define U_TABLE_SIZE 80000
#else
# define S_TABLE_SIZE	97
# define U_TABLE_SIZE	1999
#endif /* LARGE_NET */

#ifdef SOLARIS
#define bzero(x, y)       memset(x, '\0', y);
#define bcopy(x,y,z)      memcpy(y,x,z)
#endif /* SOLARIS */

/* Numerics... */
#define RPL_VERSION		351
#define RPL_TRACEOPERATOR	204
#define RPL_TRACEUSER		205
#define RPL_TRACESERVER		206
#define RPL_ENDOFTRACE		262
#define RPL_STATSCLINE		213
#define RPL_STATSNLINE		214
#define RPL_STATSILINE		215
/* #define RPL_STATSKLINE		216 - Not supporting now */
#define RPL_STATSYLINE		218
#define RPL_STATSOLINE		243
#define RPL_STATSHLINE		244
#define RPL_STATSUPTIME		242
#define RPL_ENDOFSTATS		219
#define RPL_STATSDEBUG		249

/* These are here but commented out until I can get permission to change
** netgod's SendMOTD function...
**
** #define RPL_MOTD		372
** #define RPL_MOTDSTART	375
** #define RPL_ENDOFMOTD	376
**
*/

/* Sanity checking */
#ifdef CYGWIN
# undef FNMATCH
#endif

/* Macros that were in m_stats.h */
#define DecreaseOpers(x)        x->opers--; me.opers--;
#define DecreaseUsers(x)        x->users--; me.users--;
#define DecreaseServers()       me.servers--;

/* External variables... */
extern int servsock;
extern char s_StatServ[MAXNICK], s_Spam[8];
extern const char version[];

#ifdef SUPPORT_MSG_UH
extern char s_UH[MAXUSER + MAXHOST + 2];
#endif

/* Our data structures are recursive */
typedef struct server_ Server;
typedef struct user_ User;
typedef struct stats_ Stats;
typedef struct tld_ TLD;
typedef struct myuser_ MyUser;

/* This struct is getting out of hand...
** It needs to be split into at least a netinfo and
** a settings struct.  The remaining things will be 
** in the "catch all" me struct
*/
struct me {
	char name[MAXHOST];
	int port;
	int r_time;
	char uplink[MAXHOST];
	char vhost[MAXHOST];
	char pass[MAXPASS];
	char services_name[MAXHOST];
	char desc[LINE];
	time_t t_start;
	unsigned int enable_spam : 1;
	unsigned int want_privmsg : 1;
	unsigned int enable_jupe : 1;
	unsigned int enable_quote : 1;
	unsigned int send_extreme_lag_notices : 1;
	unsigned int onlyopers : 1;
	unsigned int enable_html : 1;
	unsigned int set_oper : 1;
	unsigned int set_deaf : 1;
	unsigned int hide_services : 1;
	unsigned int flood_action : 1;
	unsigned int use_vhost : 1;
	int opers;
	int maxopers;
	int lag_time;
	long users;
	long maxusers;
	int servers;
	int maxservers;
	time_t t_maxopers;
	time_t t_maxusers;
	time_t t_maxservers;
	unsigned int away;
	Server *s;
	int requests;
	char html[BUFSIZE];
	int html_time;
	char html_link[BUFSIZE];
	char join_chans[BUFSIZE];
	char url[BUFSIZE];
	int flood;
	int flood_time;
} me;

struct StatServ {
	char user[MAXUSER];
	char host[MAXHOST];
} StatServ;

struct server_ {
	Server *next, *prev;
	char name[MAXHOST];
	char desc[LINE];
	int hops;
#ifdef IRCD_2_10
	int token;
#endif
	long hash;
	Stats *stats;
	unsigned int users;
	int opers;
	time_t connected_since;
	time_t last_announce;
	int ping;
	int lowest_ping;
	int highest_ping;
	time_t t_lowest_ping;
	time_t t_highest_ping;
	Server *uplink;
};

struct user_ {
	User *next, *prev;
	char nick[MAXNICK];
	char *hostname;
	char *username;
	Server *server;
	TLD *tld;
	MyUser *myuser;
	int flood;
	int flood_sent : 1;
	time_t t_flood;
	long hash;
	unsigned int is_oper : 1;
	unsigned int is_away : 1;
};

struct stats_ {
	Stats *next;
	char name[MAXHOST];
	int numsplits;
	long maxusers;
	time_t t_maxusers;
	int maxopers;
	time_t t_maxopers;
	time_t lastseen;
	time_t starttime;
	unsigned int operkills;
	unsigned int serverkills;
};

struct ping {
    time_t last_sent;
    int ulag;
} ping;

struct daily {
	int servers;
	time_t t_servers;
	int users;
	time_t t_users;
	int opers;
	time_t t_opers;
} daily;

struct tld_ {
	TLD *next;
	char tld[5];
	char *country;
	int users;
};

struct myuser_ {
	MyUser *next;
	char *username;
	char *password;
	time_t lastseen;
	User *u;
	unsigned int ison : 1;
};

/* sock.c */
extern int ConnectTo(char *, int);
extern void read_loop();
extern void sts(char *, ...);
extern void log(char *, ...);
extern void ResetLogs();
extern char *sctime(time_t);
extern char *sftime(time_t);
#if defined(CRYPT) || defined(MD5)
extern char *make_salt();
#endif /* CRYPT || MD5 */

/* conf.c */
extern void strip(char *);
extern void InitDefaults();
extern void ConfLoad();
extern void rehash();

/* main.c */
extern void login();
extern void init_statserv();
extern void init_spam();
extern void init_join();
extern void *smalloc(long);
extern char *sstrdup(const char *);
extern unsigned long HASH(const unsigned char *, int);
extern char *strlower(char *);

/* ircd.c */
extern void parse();
extern void privmsg(char *, const char *, char *, ...);
extern void privmsg_list(char *, char *, const char **);
extern void globops(char *, char *, ...);
extern int flood(User *);
extern void SendQuit(User *, int, char *);
extern void Trace(User *);
extern void ParseStats(User *, char);

/* timer.c */
extern void chk();
extern void TimerReset();
extern void TimerSpam();
extern void TimerPings();
extern void TimerMidnight();
extern int is_midnight();

/* statserv.c */
extern void statserv(char *);
extern void spam(char *);

/* users.c */
extern Server *serverlist[S_TABLE_SIZE];
extern User *userlist[U_TABLE_SIZE];
extern MyUser *myuhead;
#ifdef IRCD_2_10
extern void AddUser(char *, char *, char *, Server *);
#else
extern void AddUser(char *, char *, char *, char *);
#endif
extern void DelUser(char *);
extern void Change_User(User *, char *);
extern User *finduser(char *);
extern void UserMode(char *, char *);
extern void init_user_hash();
#ifdef IRCD_2_10
extern void AddServer(char *, int, int, char *, char *);
extern Server *findservtoken(int);
#else
extern void AddServer(char *, int, char *, char *);
#endif
extern void DelServer(char *);
extern Server *findserver(char *);
extern void init_server_hash();
extern void LoadMyUsers();
extern void SaveMyUsers();
extern void DeleteMyUser(char *);
extern MyUser *findmyuser(char *);

/* stats.c */
extern Stats *shead;
extern TLD *tldhead;
extern char *uptime(time_t);
extern TLD *findtld(char *);
extern TLD *AddTLD(User *);
extern void LoadTLD();
extern void init_tld();
extern void IncreaseOpers(Server *);
extern void IncreaseUsers(Server *);
extern void IncreaseServers();
extern void AddStats(Server *);
extern Stats *findstats(char *);
extern void SaveStats();
extern void LoadStats();

/* help.c */
extern const char *ss_help[];
extern const char *ss_myuser_help[];
extern const char *ss_server_help[];
extern const char *ss_map_help[];
extern const char *ss_netstats_help[];
extern const char *ss_daily_help[];
extern const char *ss_tld_help[];
extern const char *ss_tld_map_help[];
extern const char *ss_operlist_help[];
extern const char *ss_login_help[];
extern const char *ss_user_help[];
extern const char *ss_shutdown_help[];
extern const char *ss_restart_help[];
extern const char *ss_stats_help[];
extern const char *ss_logs_help[];
extern const char *ss_join_help[];
extern const char *ss_update_help[];
extern const char *ss_quote_help[];
extern const char *ss_url_help[];
extern const char *ss_status_help[];
extern const char *ss_uptime_help[];
extern const char *ss_jupe_help[];

/* html.c */
extern void DumpHTML();

#endif
