/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: statserv.c,v 1.50 2000/11/13 02:36:15 wcampbel Exp $
*/

#include "stats.h"

#ifdef LINUX
/* A kludge in mkpasswd.c, since worthless modern Linux systems require
** _XOPEN_SOURCE to be #define'ed BUT... it screws up the rest of the source
** preventing all str* functions from being defined
*/
extern char *crypt();
#endif

extern const char version_date[], version_time[];

static void ss_server(User *, char *);
static void ss_map(User *);
static void ss_uptime(User *);
static void ss_netstats(User *);
static void ss_daily(User *);
static void ss_tld(User *, char *);
static void ss_tld_map(User *);
static void ss_operlist(User *, char *, char *);
static void ss_login(User *, char *, char *);
static void ss_user(User *, char *, char *, char *);
static void ss_shutdown(User *, char *);
static void ss_restart(User *, char *);
static void ss_stats(User *, char *, char *, char *);
static void ss_logs(User *);
static void ss_jupe(User *, char *);
static void ss_join(User *, char *);
static void ss_quote(User *, char *);
static void ss_cping(User *, char *);
static void ss_cversion(User *);
static void ss_update(User *);
static void ss_url(User *);
static void ss_status(User *);

/* The big PRIVMSG parsing function */
void statserv(char *line)
{
	char *cmd, *nick = strtok(line, " ");
	User *u;

	nick++;
	cmd = strtok(NULL, " ");
	cmd = strtok(NULL, " ");
	cmd = strtok(NULL, " ");
	cmd++;
	u = finduser(nick);
	if (!u) {
		log("Unable to finduser %s (statserv)", nick);
		return;
	}

	me.requests++;

	if (flood(u))
		return;

	log("%s received message from %s: %s", s_StatServ, nick, cmd);
 
	if (me.onlyopers && !u->is_oper) {
		privmsg(u->nick, s_StatServ,
			"This service is only available to IRCops.");
		return;
	}

	if (!strcasecmp(cmd, "HELP")) {
		cmd = strtok(NULL, " ");
		if (!cmd) {
			privmsg_list(nick, s_StatServ, ss_help);
			if (u->myuser != NULL)
				privmsg_list(nick, s_StatServ, ss_myuser_help);
		} else if (!strcasecmp(cmd, "SERVER"))
			privmsg_list(nick, s_StatServ, ss_server_help);
		else if (!strcasecmp(cmd, "MAP"))
			privmsg_list(nick, s_StatServ, ss_map_help);
		else if (!strcasecmp(cmd, "NETSTATS"))
			privmsg_list(nick, s_StatServ, ss_netstats_help);
		else if (!strcasecmp(cmd, "DAILY"))
			privmsg_list(nick, s_StatServ, ss_daily_help);
		else if (!strcasecmp(cmd, "TLD"))
			privmsg_list(nick, s_StatServ, ss_tld_help);
		else if (!strcasecmp(cmd, "TLDMAP"))
			privmsg_list(nick, s_StatServ, ss_tld_map_help);
		else if (!strcasecmp(cmd, "OPERLIST"))
			privmsg_list(nick, s_StatServ, ss_operlist_help);
		else if (!strcasecmp(cmd, "URL"))
			privmsg_list(nick, s_StatServ, ss_url_help);
		else if (!strcasecmp(cmd, "STATUS"))
			privmsg_list(nick, s_StatServ, ss_status_help);
		else if (!strcasecmp(cmd, "UPTIME"))
			privmsg_list(nick, s_StatServ, ss_uptime_help);
		else if (!strcasecmp(cmd, "LOGIN"))
			privmsg_list(nick, s_StatServ, ss_login_help);
		else if (!strcasecmp(cmd, "SHUTDOWN") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_shutdown_help);
		else if (!strcasecmp(cmd, "DIE") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_shutdown_help);
		else if (!strcasecmp(cmd, "RESTART") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_restart_help);
		else if (!strcasecmp(cmd, "USER") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_user_help);
		else if (!strcasecmp(cmd, "STATS") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_stats_help);
		else if (!strcasecmp(cmd, "LOGS") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_logs_help);
		else if (!strcasecmp(cmd, "JOIN") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_join_help);
		else if (!strcasecmp(cmd, "UPDATE") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_update_help);
		else if (!strcasecmp(cmd, "JUPE") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_jupe_help);
		else if (!strcasecmp(cmd, "QUOTE") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_quote_help);
		else if (!strcasecmp(cmd, "RAW") && u->myuser)
			privmsg_list(nick, s_StatServ, ss_quote_help);
		else
			privmsg(nick, s_StatServ, "Unknown Help Topic: \2%s\2",
				cmd);
	} else if (!strcasecmp(cmd, "SERVER")) {
		cmd = strtok(NULL, " ");
		ss_server(u, cmd);
	} else if (!strcasecmp(cmd, "MAP")) {
		ss_map(u);
	} else if (!strcasecmp(cmd, "UPTIME")) {
		ss_uptime(u);
	} else if (!strcasecmp(cmd, "NETSTATS")) {
		ss_netstats(u);
	} else if (!strcasecmp(cmd, "DAILY")) {
		ss_daily(u);
	} else if (!strcasecmp(cmd, "TLD")) {
		cmd = strtok(NULL, " ");
		ss_tld(u, cmd);
	} else if (!strcasecmp(cmd, "TLDMAP")) {
		ss_tld_map(u);
	} else if (!strcasecmp(cmd, "OPERLIST")) {
		char *t;
		cmd = strtok(NULL, " ");
		t = strtok(NULL, " ");
		ss_operlist(u, cmd, t);
	} else if (!strcasecmp(cmd, "URL")) {
		ss_url(u);
	} else if (!strcasecmp(cmd, "STATUS")) {
		ss_status(u);
	} else if (!strcasecmp(cmd, "LOGIN")) {
		char *t;
		cmd = strtok(NULL, " ");
		t = strtok(NULL, " ");
		ss_login(u, cmd, t);
	} else if (!strcasecmp(cmd, "USER")) {
		char *t, *m;
		m = strtok(NULL, " ");
		cmd = strtok(NULL, " ");
		t = strtok(NULL, " ");
		ss_user(u, m, cmd, t);
	} else if (!strcasecmp(cmd, "SHUTDOWN")) {
		cmd = strtok(NULL, "\n");
		ss_shutdown(u, cmd);
	} else if (!strcasecmp(cmd, "DIE")) {
		cmd = strtok(NULL, "\n");
		ss_shutdown(u, cmd);
	} else if (!strcasecmp(cmd, "RESTART")) {
		cmd = strtok(NULL, "\n");
		ss_restart(u, cmd);
	} else if (!strcasecmp(cmd, "STATS")) {
		char *t, *m;
		m = strtok(NULL, " ");
		cmd = strtok(NULL, " ");
		t = strtok(NULL, " ");
		ss_stats(u, m, cmd, t);
	} else if (!strcasecmp(cmd, "LOGS")) {
		ss_logs(u);
	} else if (!strcasecmp(cmd, "JUPE")) {
		cmd = strtok(NULL, " ");
		ss_jupe(u, cmd);
	} else if (!strcasecmp(cmd, "JOIN")) {
		cmd = strtok(NULL, " ");
		ss_join(u, cmd);
	} else if (!strcasecmp(cmd, "QUOTE")) {
		cmd = strtok(NULL, "\n");
		ss_quote(u, cmd);
	} else if (!strcasecmp(cmd, "RAW")) {
		cmd = strtok(NULL, "\n");
		ss_quote(u, cmd);
	} else if (!strcasecmp(cmd, "\001PING")) {
		cmd = strtok(NULL, "\001");
		ss_cping(u, cmd);
        } else if (!strcasecmp(cmd, "\001VERSION\001")) {
		ss_cversion(u);
	} else if (!strcasecmp(cmd, "UPDATE")) {
		ss_update(u);
	} else if (cmd[0] == '\001') {
		/* NO OP - Ignore all other CTCP's */
        } else {
		privmsg(nick, s_StatServ, "Unknown Command: \2%s\2",
			cmd);
	}
}

/* process messages sent to the spam detecter */
void spam(char *line)
{
	char *rest, *nick = strtok(line, " ");
	User *u;

	nick++;
	rest = strtok(NULL, " ");
	rest = strtok(NULL, " ");
	rest = strtok(NULL, "");
	rest++;
	u = finduser(nick);
	if (!u) {
		log("Unable to find user %s (spam)", nick);
		return;
	}

	if (flood(u))
		return;

	if (u->is_oper)
		return;

	globops(me.name, "Possible Mass Message -\2(%s!%s@%s)\2- %s", u->nick,
		u->username, u->hostname, rest);
	log("Possible Mass Message -(%s!%s@%s)- %s", u->nick, u->username,
		u->hostname, rest);
}

static void ss_server(User *u, char *server)
{
	Stats *st;
	Server *s;

	if (!server) {
		privmsg(u->nick, s_StatServ, "Syntax: /msg %s SERVER <server name>",
			s_StatServ);
		privmsg(u->nick, s_StatServ, "For additional help, /msg %s HELP",
			s_StatServ);
		return;
	}

	st = findstats(server);
	s = findserver(server);

	if (!st) {
		privmsg(u->nick, s_StatServ, "No information available on %s",
			server);
		return;
	}

	/* If we want services to be hidden, catch it here */
	if (me.hide_services && (!strcasecmp(server, me.services_name))) {
		privmsg(u->nick, s_StatServ, 
		    "No information available on %s (hidden server)", server);
		return;
	}

	privmsg(u->nick, s_StatServ, "Statistics for %s since %s", server,
		sftime(st->starttime));
	if (!s)
		privmsg(u->nick, s_StatServ, "Server Last Seen:  %s",
			sftime(st->lastseen));
	if (s)
		privmsg(u->nick, s_StatServ, "Current Users: %-3ld (%.0f%%)", s->users,
			(float)s->users / (float)me.users * 100);
	privmsg(u->nick, s_StatServ, "Maximum Users: %-3ld %s", st->maxusers,
		sftime(st->t_maxusers));
	if (s)
		privmsg(u->nick, s_StatServ, "Current Opers: %d", s->opers);
	privmsg(u->nick, s_StatServ, "Maximum Opers: %-3d %s", st->maxopers,
		sftime(st->t_maxopers));
	privmsg(u->nick, s_StatServ, "  IRCop Kills: %d", st->operkills);
	privmsg(u->nick, s_StatServ, " Server Kills: %d", st->serverkills);
	if (s) {
	privmsg(u->nick, s_StatServ, "  Lowest Ping: %-3d %s", s->lowest_ping,
		sftime(s->t_lowest_ping));
	privmsg(u->nick, s_StatServ, " Highest Ping: %-3d %s", s->highest_ping,
		sftime(s->t_highest_ping));
	privmsg(u->nick, s_StatServ, " Current Ping: %d", s->ping);
	}
	if (st->numsplits >= 1)
	privmsg(u->nick, s_StatServ, "%s has split from the network %d time%s.",
		st->name, st->numsplits, (st->numsplits == 1) ? "" : "s");
	else
	privmsg(u->nick, s_StatServ, "%s has never split from the network.",
		st->name);
	privmsg(u->nick, s_StatServ, "End of Statistics.");
}

static void ss_map(User *u)
{
	Server *s;
	register int i;

	privmsg(u->nick, s_StatServ, "Network Map [\2NAME\2] [\2USERS/MAX\2] "
		"[\2OPERS/MAX\2] [\2PERCENT\2] [\2LAG\2]");
	for (i = 0; i < S_TABLE_SIZE; i++) {
		for (s = serverlist[i]; s; s = s->next) {
		    if (me.hide_services) {
			/* meaning if the server does NOT match the services
			** server, therefore show it
			*/
                        if (strcasecmp(s->name, me.services_name)) {
			    /* I should also check for juped servers too - 
			    ** servers that have services as an uplink
			    */
			    privmsg(u->nick, s_StatServ, "%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%] %d",s->name,
				s->users, s->stats->maxusers, 
				s->opers, s->stats->maxopers,
				(float) s->users / (float) me.users * 100,
				s->ping);
		        }
		    } else {
			    privmsg(u->nick, s_StatServ, "%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%] %d",s->name,
				s->users, s->stats->maxusers,
                                s->opers, s->stats->maxopers,
                                (float) s->users / (float) me.users * 100,
                                s->ping);
                    }
		}
	}
	privmsg(u->nick, s_StatServ, "%-30s   [%2d/%2d] [%2d/%2d] [100%%] x", "\2Totals:\2", me.users, me.maxusers, me.opers, me.maxopers);
	privmsg(u->nick, s_StatServ, "End of Map.");
}

static void ss_status(User *u)
{
	privmsg(u->nick, s_StatServ, "Statistics Status:");
	privmsg(u->nick, s_StatServ, "Reconnect Time: %d", me.r_time);
	privmsg(u->nick, s_StatServ, "Extreme Lag Notices: %s",
		(me.send_extreme_lag_notices) ? "Enabled" : "Disabled");
        if (me.send_extreme_lag_notices)
		privmsg(u->nick, s_StatServ, "Lag notices sent for lag greater than %d seconds", me.lag_time);
	privmsg(u->nick, s_StatServ, "Oper Only: %s",
		(me.onlyopers) ? "Enabled" : "Disabled");
	privmsg(u->nick, s_StatServ, "HTML Dump: %s",
		(me.enable_html) ? "Enabled" : "Disabled");
	if (me.enable_html) {
		privmsg(u->nick, s_StatServ, "HTML File: %s", me.html);
		privmsg(u->nick, s_StatServ, "HTML File created every %d seconds", me.html_time);
		privmsg(u->nick, s_StatServ, "URL: %s", me.url);
	}
	privmsg(u->nick, s_StatServ, "Hide Services: %s", 
		(me.hide_services) ? "Enabled" : "Disabled");
	if (me.hide_services) 
		privmsg(u->nick, s_StatServ, "Services Name: %s", 
			me.services_name);
	/* me.name, me.desc, me.uplink all redundant, as they can be seen 
	** from LINKS or from whois s_StatServ
	*/
	privmsg(u->nick, s_StatServ, "Set to oper: %s",
		(me.set_oper) ? "Enabled" : "Disabled");

    if (u->myuser) {
        privmsg(u->nick, s_StatServ, "Spam Service: %s",
                (me.enable_spam) ? "Enabled" : "Disabled");
	privmsg(u->nick, s_StatServ, "JUPE: %s",
		(me.enable_jupe) ? "Enabled" : "Disabled");
	privmsg(u->nick, s_StatServ, "QUOTE: %s",
		(me.enable_quote) ? "Enabled" : "Disabled");
        if (me.flood) {
		privmsg(u->nick, s_StatServ, 
			"Flood Control: %d requests in %d seconds", me.flood, 
			me.flood_time);
		privmsg(u->nick, s_StatServ, "Flood Action: %s", 
			(me.flood_action) ? "IGNORE" : "KILL");
	}
    }
}

static void ss_uptime(User *u)
{
	privmsg(u->nick, s_StatServ, "Uptime and Memory Statistics:");
	privmsg(u->nick, s_StatServ, "%s", uptime(me.t_start));
	privmsg(u->nick, s_StatServ, "Entries in User Hash: %ld", me.users);
	privmsg(u->nick, s_StatServ, "Entries in Server Hash: %d", me.servers);
	privmsg(u->nick, s_StatServ, "Statistic Requests: %d", me.requests);
	privmsg(u->nick, s_StatServ, "User Memory Usage: %d bytes",
		sizeof(User) * me.users);
	privmsg(u->nick, s_StatServ, "Server Memory Usage: %d bytes",
		sizeof(Server) * me.servers);
	privmsg(u->nick, s_StatServ, "End of Information.");
}

static void ss_netstats(User *u)
{
	privmsg(u->nick, s_StatServ, "Network Statistics:");
	privmsg(u->nick, s_StatServ, "   Current Users: %ld", me.users);
	privmsg(u->nick, s_StatServ, "   Maximum Users: %-2ld %s", me.maxusers,
		sftime(me.t_maxusers));
	privmsg(u->nick, s_StatServ, "   Current Opers: %d", me.opers);
	privmsg(u->nick, s_StatServ, "   Maximum Opers: %-2d %s", me.maxopers,
		sftime(me.t_maxopers));
	privmsg(u->nick, s_StatServ, "  Users Set Away: %-2d", me.away);
	privmsg(u->nick, s_StatServ, " Current Servers: %d", me.servers);
	privmsg(u->nick, s_StatServ, " Maximum Servers: %-2d %s", me.maxservers,
		sftime(me.t_maxservers));
	privmsg(u->nick, s_StatServ, "End of Information.");
}

static void ss_daily(User *u)
{
	privmsg(u->nick, s_StatServ, "Daily Network Statistics:");
	privmsg(u->nick, s_StatServ, " ");
	privmsg(u->nick, s_StatServ, "Maximum Servers: %-2d %s", daily.servers,
		sftime(daily.t_servers));
	privmsg(u->nick, s_StatServ, "  Maximum Users: %-2d %s", daily.users,
		sftime(daily.t_users));
	privmsg(u->nick, s_StatServ, "  Maximum Opers: %-2d %s", daily.opers,
		sftime(daily.t_opers));
	privmsg(u->nick, s_StatServ, "All daily statistics are reset at midnight.");
	privmsg(u->nick, s_StatServ, "End of Information.");
}

static void ss_tld(User *u, char *tld)
{
	TLD *tmp;

	if (!tld) {
		privmsg(u->nick, s_StatServ, "Syntax: /msg %s TLD <tld>", s_StatServ);
		privmsg(u->nick, s_StatServ, "For additional help, /msg %s HELP", 
			s_StatServ);
		return;
	}

	if (*tld == '*')
		tld++;
	if (*tld == '.')
		tld++;

	tmp = findtld(tld);

	if (!tmp)
		privmsg(u->nick, s_StatServ, "Top Level Domain \2%s\2 does not exist.",
			tld);
	else
		privmsg(u->nick, s_StatServ, "%s", tmp->country);
}

static void ss_tld_map(User *u)
{
	TLD *t;

	privmsg(u->nick, s_StatServ, "Top Level Domain Statistics:");
	for (t = tldhead; t; t = t->next) {
		if (t->users != 0)
		privmsg(u->nick, s_StatServ, "%3s \2%3d\2 (%2.0f%%) -> %s", t->tld,
			t->users, (float)t->users / (float)me.users * 100, t->country);
	}
	privmsg(u->nick, s_StatServ, "End of Information.");
}

static void ss_operlist(User *origuser, char *flags, char *server)
{
	register int i, j = 0;
	int away = 0;
	register User *u;

	if (flags && !strcasecmp(flags, "NOAWAY")) {
		away = 1;
		flags = NULL;
	}
	if (!away && flags && strchr(flags, '.'))
		server = flags;

	privmsg(origuser->nick, s_StatServ, "On-Line IRCops:");

	for (i = 0; i < U_TABLE_SIZE; i++) {
		for (u = userlist[i]; u; u = u->next) {
			if (away && u->is_away)
				continue;
			if (me.hide_services && 
			    !strcasecmp(u->server->name, me.services_name))
				continue;
			if (u->is_oper != 1)
				continue;
			if (!server) {
				if (u->is_oper != 1)	continue;
				j++;
				privmsg(origuser->nick, s_StatServ, "[%2d] %-15s %s",j, u->nick,
					u->server->name);
				continue;
			} else {
				if (strcasecmp(server, u->server->name))	continue;
				if (u->is_oper != 1)	continue;
				j++;
				privmsg(origuser->nick, s_StatServ, "[%2d] %-15s %s",j, u->nick,
					u->server->name);
				continue;
			}
		}
	}
	privmsg(origuser->nick, s_StatServ, "End of Listing.");
}

static void ss_login(User *u, char *user, char *pass)
{
	MyUser *myu;
#if defined(CRYPT) || defined(MD5)
	char * crpass;
#endif

	if (!user || !pass) {
		privmsg(u->nick, s_StatServ, "Syntax: /msg %s LOGIN \37username\37"
			" \37password\37", s_StatServ);
		return;
	}
	myu = findmyuser(user);
	if (!myu) {
		privmsg(u->nick, s_StatServ, "Unknown User.");
		log("Failed login attempt (unknown user (%s))", user);
		return;
	}
#if defined(CRYPT) || defined(MD5)
	/* Passing in the whole password is fine, no need to rip of the
	** salt.  This actually will support md5, but has not been tested
	*/
	crpass = crypt(pass, myu->password);
	if (!strcmp(myu->username, user) && !strcmp(myu->password, crpass)) {
#else
	if (!strcmp(myu->username, user) && !strcmp(myu->password, pass)) {
#endif /* CRYPT || MD5 */
		log("%s [%s](%s) now walks among the gods.", u->nick, myu->username,
			u->hostname);
		u->myuser = myu;
		privmsg(u->nick, s_StatServ, "Permission Granted.");
		privmsg(u->nick, s_StatServ, "You now walk among the gods.");
		myu->lastseen = time(NULL);
		myu->ison = 1;
		/* We keep the CURRENT person who LOGIN'ed through that
		** access entry pointed to by the u pointer.  This is used
		** to obtain information on the user who logged in, ie. 
		** in STATS p
		*/
		myu->u = u;
		return;
	} else {
		log("%s [%s](%s) failed login.", u->nick, myu->username, u->hostname);
		privmsg(u->nick, s_StatServ, "Access Denied.");
	}
}

static void ss_user(User *u, char *what, char *user, char *pass)
{
	int a = 0;

	if (!what) {
		privmsg(u->nick, s_StatServ, "Syntax: /msg %s USER [ADD|DEL|LIST]",
			s_StatServ);
		privmsg(u->nick, s_StatServ, "For additional help, /msg %s HELP",
			s_StatServ);
		return;
	}

	if (!u->myuser) {
		log("Denied access (USER) to %s [%s].", u->nick, u->hostname);
		privmsg(u->nick, s_StatServ, "Access Denied.");
		return;
	}

	if (!strcasecmp(what, "LIST"))
		a = 0;
	else if (!strcasecmp(what, "ADD"))
		a = 1;
	else if (!strcasecmp(what, "DEL"))
		a = 2;
	else {
		privmsg(u->nick, s_StatServ, "Unknown Flag: %s", what);
		return;
	}

	switch (a) {
		case 0: {
			MyUser *myu;
			privmsg(u->nick, s_StatServ, "UserList:");
			for (myu = myuhead; myu; myu = myu->next) {
				privmsg(u->nick, s_StatServ, "%-20s (%s) -> %s", myu->username,
					(myu->ison == 0) ? "offline" : " online",
					(myu->lastseen == 0) ? "Never Seen":sftime(myu->lastseen));
			}
			privmsg(u->nick, s_StatServ, "End of List.");
			break;
		}
		case 1: {
			MyUser *myu;
#if defined(CRYPT) || defined(MD5)
			char * crpass;
			char * salt;
#endif /* CRYPT || MD5 */
			if (!user || !pass) {
				privmsg(u->nick, s_StatServ, "Syntax: /msg %s USER ADD "
					"<username> <password>", s_StatServ);
				break;
			}
			myu = smalloc(sizeof(MyUser));
			myu->username = strdup(user);
#if defined(CRYPT) || defined(MD5)
			salt = make_salt();
			crpass = crypt(pass, salt);
			myu->password = strdup(crpass);
			free(salt);
			/* if I free(crpass), I'll get runtime errors */
#else 
			myu->password = strdup(pass);
#endif /* CRYPT */
			myu->lastseen = time(NULL);
			if (!myuhead) {
				myuhead = myu;
				myuhead->next = NULL;
			} else {
				myu->next = myuhead;
				myuhead = myu;
			}
			privmsg(u->nick, s_StatServ, "Added user %s with password %s",
				user, pass);
			log("%s [%s](%s) added user %s", u->nick, u->myuser->username,
				u->hostname, myu->username);
			SaveMyUsers();
			break;
		}
		case 2: {
			MyUser *myu;
			if (!user) {
				privmsg(u->nick, s_StatServ, "Syntax: /msg %s USER DEL <username>",
					s_StatServ);
				break;
			}
			if ((myu = findmyuser(user)) == NULL) {
				privmsg(u->nick, s_StatServ, "Unable to find user %s",
					user);
				break;
			}
			DeleteMyUser(user);
			SaveMyUsers();
			privmsg(u->nick, s_StatServ, "Deleted user %s", user);
			log("%s [%s](%s) deleted user %s", u->nick, u->myuser->username,
				u->hostname, user);
			break;
		}
	}
}

static void ss_shutdown(User *u, char *comm)
{
	if (!u->myuser) {
		log("Access Denied (SHUTDOWN) to %s", u->nick);
		privmsg(u->nick, s_StatServ, "Access Denied.");
		return;
	}
	/* Yes, we are very nasty like this. */
	globops(me.name, "%s requested SHUTDOWN: %s", u->nick,
                (comm) ? comm : "No Reason");
	SendQuit(u, 0, comm);
	sleep(1);
	close(servsock);
	SaveStats();
	log("%s [%s](%s) requested SHUTDOWN: %s", u->nick, u->myuser->username,
		u->hostname, (comm) ? comm : "No Reason");
	exit(0);
}

static void ss_restart(User *u, char *comm)
{
        if (!u->myuser) {
                log("Access Denied (RESTART) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
                return;
        }
        /* Yes, we are very nasty like this. */
        globops(me.name, "%s requested RESTART: %s", u->nick,
                (comm) ? comm : "No Reason");
        SendQuit(u, 1, comm);
	sleep(1);
        close(servsock);
        SaveStats();
        log("%s [%s](%s) requested RESTART: %s", u->nick, u->myuser->username,
                u->hostname, (comm) ? comm : "No Reason");
	execv(SPATH, NULL);
}

static void ss_stats(User *u, char *cmd, char *arg, char *arg2)
{
	Stats *st;

	if (!u->myuser) {
		log("Access Denied (STATS) to %s", u->nick);
		privmsg(u->nick, s_StatServ, "Access Denied.");
		return;
	}

	if (!cmd) {
		privmsg(u->nick, s_StatServ, "Syntax: /msg %s STATS [DEL|LIST|COPY]",
			s_StatServ);
		privmsg(u->nick, s_StatServ, "For additional help, /msg %s HELP",
			s_StatServ);	
		return;
	}
	if (!strcasecmp(cmd, "LIST")) {
		int i = 1;
		privmsg(u->nick, s_StatServ, "Statistics Database:");
		for (st = shead; st; st = st->next) {
			privmsg(u->nick, s_StatServ, "[%-2d] %s", i, st->name);
			i++;
		}
		privmsg(u->nick, s_StatServ, "End of List.");
		log("%s requested STATS LIST.", u->nick);
	} else if (!strcasecmp(cmd, "DEL")) {
		Stats *m = NULL;
		if (!arg) {
			privmsg(u->nick, s_StatServ, "Syntax: /msg %s STATS DEL <name>",
				s_StatServ);
			privmsg(u->nick, s_StatServ, "For additonal help, /msg %s HELP",
				s_StatServ);
			return;
		}
		st = findstats(arg);
		if (!st) {
			privmsg(u->nick, s_StatServ, "%s is not in the database!",
				arg);
			return;
		}
		for (st = shead; st; st = st->next) {
			if (!strcasecmp(arg, st->name))
				break;
			m = st;
		}
		if (m)
			m->next = st->next;
		else
			shead = st->next;
		free(st);
		privmsg(u->nick, s_StatServ, "Removed %s from the database.", arg);
		log("%s requested STATS DEL %s", u->nick, arg);
	} else if (!strcasecmp(cmd, "COPY")) {
		Server *s;

		if (!arg || !arg2) {
			privmsg(u->nick, s_StatServ, "Syntax: /msg %s STATS COPY <name> "
				" <newname>", s_StatServ);
			return;
		}
		st = findstats(arg2);
		if (st)
			free(st);

		st = findstats(arg);
		if (!st) {
			privmsg(u->nick, s_StatServ, "No entry in the database for %s",
				arg);
			return;
		}
		s = findserver(arg);
		if (s) {
			privmsg(u->nick, s_StatServ, "Server %s is online!", arg);
			return;
		}
		s = NULL;
		memcpy(st->name, arg2, sizeof(st->name));
		s = findserver(st->name);
		if (s)
			s->stats = st;
		privmsg(u->nick, s_StatServ, "Moved database entry for %s to %s",
			arg, arg2);
		log("%s requested STATS COPY %s -> %s", u->nick, arg, arg2);
	} else {
		privmsg(u->nick, s_StatServ, "Invalid Argument.");
		privmsg(u->nick, s_StatServ, "For help, /msg %s HELP", s_StatServ);
	}
}

static void ss_logs(User *u)
{
#ifndef DEBUG
	FILE *fp;
	char buf[512];
#endif

	if (!u->myuser) {
		log("Access Denied (LOGS) to %s", u->nick);
		privmsg(u->nick, s_StatServ, "Access Denied.");
		return;
	}

	log("%s requested LOGS", u->nick);
#ifdef DEBUG
	privmsg(u->nick, s_StatServ, "This command is disabled while in DEBUG.");
#else
	fp = fopen(LOGFILE, "r");
	if (!fp) {
		privmsg(u->nick, s_StatServ, "Unable to open %s", LOGFILE);
		return;
	}
	while (fgets(buf, sizeof(buf), fp)) {
		buf[strlen(buf)] = '\0';
		privmsg(u->nick, s_StatServ, "%s", buf);
	}
	fclose(fp);
#endif
}

static void ss_jupe(User *u, char *server)
{
	/* NOTE:  should I prevent the main services server from
	** being juped here?
	** Also prevent my uplink from being JUPED.
	** JUPE is a touchy feature anyway...
	**
	** ALSO.... We should maybe make JUPE's more perminant...
	** Or just remove JUPE functionality completely and use 
	** an updated jupe.c/jupe.pl or main services.
	*/

	if (!u->is_oper) {
		privmsg(u->nick, s_StatServ, "Access Denied.");
		return;
	}
	if (!me.enable_jupe) {
		privmsg(u->nick, s_StatServ, "JUPE Disabled.");
		return;
	}
	if (!server) {
		privmsg(u->nick, s_StatServ, "Syntax: /msg %s JUPE <server>",
		s_StatServ);
		return;
	}
	sts(":%s SERVER %s 1 :[Jupitered] by %s", me.name, server, u->nick);
	log("%s!%s@%s jupitered %s", u->nick, u->username, u->hostname, server);
}

static void ss_join(User *u, char *chan)
{
	if (!u->myuser) {
		log("Access Denied (JOIN) to %s", u->nick);
		privmsg(u->nick, s_StatServ, "Access Denied.");
		return;
	}

	if (chan) {
		if (chan[0] != '#') {
			privmsg(u->nick, s_StatServ, 
				"Please enter a valid channel");
			return;
		}

#if defined(IRCD_DAL)
			sts(":%s JOIN %s", s_StatServ, chan);
#else
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6)
			sts(":%s SJOIN %ld %s + :@%s", me.name, time(NULL),
				chan, s_StatServ);
#else
#if defined(IRCD_MLN)
			sts(":%s JOIN %s", s_StatServ, chan);
			sts(":%s MODE %s +o %s %ld", me.name, chan,
				s_StatServ, time(NULL));
#else
#if defined(IRCD_2_10)
			sts(":%s NJOIN %s :@%s", me.name, chan, s_StatServ);
#endif
#endif
#endif
#endif

		log("%s!%s@%s made me join %s", u->nick, u->username, 
			u->hostname, chan);

		return;
	} else {
		privmsg(u->nick, s_StatServ, "Please enter a valid channel");
		return;
	}
}

static void ss_quote(User *u, char *command)
{
	if (!u->myuser) {
		log("Access Denied (QUOTE) to %s", u->nick);
		privmsg(u->nick, s_StatServ, "Access Denied.");
		return;
	}

        if (!me.enable_quote) {
                privmsg(u->nick, s_StatServ, "QUOTE/RAW Disabled.");
                return;
        }

	if (!command) {
		privmsg(u->nick, s_StatServ, "Please enter a valid command");
		return;
	}


	sts(command);
	log("%s!%s@%s made me execute %s", u->nick, u->username, u->hostname, command);
}

static void ss_cping(User *u, char *command)
{
	if (!command) 
		return;
	sts(":%s NOTICE %s :\001PING %s\001", s_StatServ, u->nick, command);
	log("CTCP PING from %s!%s@%s", u->nick, u->username, u->hostname);
}

static void ss_cversion(User *u)
{
	sts(":%s NOTICE %s :\001VERSION %s %s %s %s\001", s_StatServ, u->nick, 
	     version, me.name, version_date, version_time);
	log("CTCP VERSION from %s!%s@%s", u->nick, u->username, u->hostname);
}

static void ss_update(User *u)
{      
        if (!u->myuser) {
                log("Access Denied (UPDATE) to %s", u->nick);
                privmsg(u->nick, s_StatServ, "Access Denied.");
                return; 
        }                       

	if (!me.enable_html) {
		privmsg(u->nick, s_StatServ, "HTML Update Disabled");
		return;
	}

	DumpHTML();
	privmsg(u->nick, s_StatServ, "HTML Update Forced");

        log("%s!%s@%s forces an HTML Update", u->nick, u->username, u->hostname);
}

static void ss_url(User *u)
{
	if (!me.enable_html) {
		privmsg(u->nick, s_StatServ, "HTML Dump Disabled");
		return;
	}

	privmsg(u->nick, s_StatServ, "HTML Status URL: %s", me.url);

	log("%s!%s@%s Looks up URL", u->nick, u->username, u->hostname);
}

