/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: timer.c,v 1.15 2000/11/10 20:49:01 wcampbel Exp $
*/

#include "stats.h"

static time_t last_spam;
static time_t last_stats_save;
static time_t last_lag_check;
static time_t last_html;
static int midnight = 0;
static void TimerLagCheck();

/* Timer check, runs every time it gets a message from the server
** BEFORE it parses the message.  This will run at LEAST every time
** that the uplink sends a PING message (look in the Y line, ping 
** timings (2nd field, after the class))
**
** This function is one that I'm sure is a bottleneck...
** It gets called every time that it receives a message from the server...
** That means unless the ircd supports user mode +d and the service is set
** user mode +d, it will call chk EVERY time someone sends a JOIN/PART/NOTICE
** or a PRIVMSG to the channel.  
*/
void chk()
{
	time_t current = time(NULL);

	if (me.enable_spam && current - last_spam > 300)
		TimerSpam();
	if (current - last_stats_save > 120) {
		SaveStats();
		last_stats_save = time(NULL);
	}
	if (current - ping.last_sent > 60) {
		TimerPings();
		ping.last_sent = time(NULL);
	}
	if (current - last_lag_check > 70) {
		TimerLagCheck();
		last_lag_check = time(NULL);
	}
	if (me.enable_html && current - last_html > me.html_time) {
		DumpHTML();
		last_html = time(NULL);
	}
	if (is_midnight() == 1 && midnight == 0) {
		TimerMidnight();
		midnight = 1;
	} else {
		if (midnight == 1 && is_midnight() == 0)
			midnight = 0;
	}
}

/* Reset the timers, called when the program is started
** Note the 30 second delay in last_html.  That enables 
** the service to sync at least partly before dumping out
** an HTML file, therefore making sure that you have at least
** part of your networks statistics to dump to HTML.  Without
** this delay, the dump that's created when you start the
** service only contains the stats service statistics
*/
void TimerReset()
{
	time_t current = time(NULL);
	last_spam = current;
	last_stats_save = current;
	last_lag_check = current;
	last_html = current - me.html_time + 30;
}

/* Create a random nick and change the spam detecter to that nick
** There should be better checking here to see if that nick already
** exists or not.
*/
void TimerSpam()
{
	register char c;
	register int i;
	char *t;
	User *u = finduser(s_Spam);

	last_spam = time(NULL);

	t = sstrdup(s_Spam);

	bzero(s_Spam, sizeof(s_Spam));
	for (i = 0; i < sizeof(s_Spam-1); i++) {
		c = (((u_short) random()) % 26) + 97;
		s_Spam[i] = c;
	}
	if (servsock > 0) {
#if defined(IRCD_DAL) || defined(IRCD_MLN) || defined(IRCD_2_10)
		    sts(":%s NICK %s", t, s_Spam);
#else
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6)
 	        sts(":%s NICK %s :%ld", t, s_Spam, time(NULL));
#endif
#endif
		Change_User(u, s_Spam);
	}

	free(t);
}

/* Send out the lag detecting PING's */
void TimerPings()
{
	register int i;
	Server *s;

#ifdef DEBUG
	log("Sendings pings...");
#endif
	ping.ulag = 0;

	for (i = 0; i < S_TABLE_SIZE; i++) {
		for (s = serverlist[i]; s; s = s->next) {
			if (!strcmp(me.name, s->name)) {
				s->ping = 0;
				continue;
			}
			sts(":%s PING %s :%s", me.name, me.name, s->name);
		}
	}
}

/* Announce lagged servers */
static void TimerLagCheck()
{
	Server *s;
	register int i;

	if (me.send_extreme_lag_notices == 0)
		return;

	/* Do a check to see if our uplink is lagged...
	** If it is, get out of here
	*/
	if (me.s) {
	    if (me.s->ping > me.lag_time)
		return;
	}

	for (i = 0; i < S_TABLE_SIZE; i++) {
		for (s = serverlist[i]; s; s = s->next) {
			if (s->ping > me.lag_time 
			    && time(NULL) - s->last_announce > 30 ) {
				globops(me.name, "The current network lag for server %s"
					" is \2%d\2", s->name, s->ping);
				s->last_announce = time(NULL);
			}
		}
	}
}

/* Process the Midnight rolling over */
void TimerMidnight()
{
#ifdef DEBUG
	log("Its midnight!!! -> %s", sctime(time(NULL)));
#endif
	/* Reset Daily Statistics */
	daily.servers = me.servers;
	daily.t_servers = time(NULL);
	daily.users = me.users;
	daily.t_users = time(NULL);
	daily.opers = me.opers;
	daily.t_opers = time(NULL);
	/* end */
	ResetLogs();
}

/* Is it Midnight (or somewhere close since chk isn't exact) */
int is_midnight()
{
	time_t current = time(NULL);
	struct tm *ltm = localtime(&current);

	if (ltm->tm_hour == 0)
		return 1;

	return 0;	
}
