/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: users.c,v 1.22 2000/11/12 23:26:33 wcampbel Exp $
*/

#include "stats.h"

#ifdef FNMATCH
# include <fnmatch.h>
#endif /* FNMATCH */

/* These are ugly functions, gets right down to the hash tables and
** manipulation of raw data
*/

static Server *new_server(char *, char *);
Server *serverlist[S_TABLE_SIZE];

MyUser *myuhead;

static User *new_user(char *);
static void add_user_to_hash_table(char *, User *);
static void del_user_from_hash_table(char *, User *);
static void add_server_to_hash_table(char *, Server *);
static void del_server_from_hash_table(char *, Server *);

User *userlist[U_TABLE_SIZE];

static void add_user_to_hash_table(char *nick, User *u)
{
	u->hash = HASH(nick, U_TABLE_SIZE);
	u->next = userlist[u->hash];
	userlist[u->hash] = (void *)u;
}

static void del_user_from_hash_table(char *nick, User *u)
{
	User *tmp, *prev = NULL;

	for (tmp = userlist[u->hash]; tmp; tmp = tmp->next) {
		if (tmp == u) {
			if (prev)
				prev->next = tmp->next;
			else
				userlist[u->hash] = tmp->next;
			tmp->next = NULL;
			return;
		}
		prev = tmp;
	}
}

static void add_server_to_hash_table(char *name, Server *s)
{
	s->hash = HASH(name, S_TABLE_SIZE);
	s->next = serverlist[s->hash];
	serverlist[s->hash] = (void *)s;
}

static void del_server_from_hash_table(char *name, Server *s)
{
	Server *tmp, *prev = NULL;

	for (tmp = serverlist[s->hash]; tmp; tmp = tmp->next) {
		if (tmp == s) {
			if (prev)
				prev->next = tmp->next;
			else
				serverlist[s->hash] = tmp->next;
			tmp->next = NULL;
			return;
		}
		prev = tmp;
	}
}

static User *new_user(char *nick)
{
	User *u;

	u = smalloc(sizeof(User));
	if (!nick)
		nick = "";
	memcpy(u->nick, nick, MAXNICK);
	add_user_to_hash_table(nick, u);

	return u;
}

#ifdef IRCD_2_10
void AddUser(char *nick, char *user, char *host, Server *server)
#else
void AddUser(char *nick, char *user, char *host, char *server)
#endif
{
	User *u;

#ifdef DEBUG
#ifdef IRCD_2_10
	log("AddUser(): %s (%s@%s) -> %s", nick, user, host, server->name);
#else
	log("AddUser(): %s (%s@%s) -> %s", nick, user, host, server);
#endif
#endif
	u = finduser(nick);
	if (u) {
		log("trying to add a user that already exists? (%s)", nick);
		return;
	}

	u = new_user(nick);
	u->hostname = sstrdup(host);
	u->username = sstrdup(user);
#ifdef IRCD_2_10
	u->server = server;
#else
	u->server = findserver(server);
#endif
	u->is_oper = 0;
	u->t_flood = time(NULL);
	u->flood = 0;
	u->is_away = 0;
	u->myuser = NULL;
	u->tld = AddTLD(u);

	if (!u->server)
		log("Unable to find server for user %s (%s?)", nick, server);
	else
		IncreaseUsers(u->server);
}

void DelUser(char *nick)
{
	User *u = finduser(nick);

#ifdef DEBUG
	log("DelUser(%s)", nick);
#endif

	if (!u) {
		log("DelUser(%s) failed!", nick);
		return;
	}

	del_user_from_hash_table(nick, u);

	DecreaseUsers(u->server);

	if (u->tld)
		u->tld->users--;
	if (u->is_away)
		me.away--;
	if (u->is_oper) {
		me.opers--;
		u->server->opers--;
	}
	if (u->myuser)
		u->myuser->ison = 0;

	free(u->hostname);
	free(u->username);
	free(u);
}

void Change_User(User *u, char *newnick)
{
#ifdef DEBUG
	log("Change_User(%s, %s)", u->nick, newnick);
#endif

	del_user_from_hash_table(u->nick, u);

	u->nick[1] = 0;
	memcpy(u->nick, newnick, MAXNICK);

	add_user_to_hash_table(u->nick, u);
}

User *finduser(char *nick)
{
	User *u;

#ifdef DEBUG
	log("finduser(%s)", nick);
#endif

	u = userlist[HASH(nick, U_TABLE_SIZE)];
	while (u && strcasecmp(u->nick, nick) != 0)
		u = u->next;

#ifdef DEBUG
	log("finduser(%s) -> %s", nick, (u) ? u->nick : "NOTFOUND");
#endif

	return u;
}

void init_user_hash()
{
	int i;
	User *u, *prev;

	for (i = 0; i < U_TABLE_SIZE; i++) {
		u = userlist[i];
		while (u) {
			prev = u->next;
#ifdef DEBUG
	log("init_user_hash() deleting %s", u->nick);
#endif
			free(u->hostname);
			free(u->username);
			free(u);

			u = prev;
		}
		userlist[i] = NULL;
	}
	bzero((char *)userlist, sizeof(userlist));
}

void UserMode(char *nick, char *modes)
{
	User *u = finduser(nick);
	int add = 0;

	if (!u) {
		log("Changing modes for unknown user: %s", nick);
		return;
	}

	/* while (*modes++) { */
	while (*modes) {
		switch(*modes) {
			case '+': add = 1;	break;
			case '-': add = 0;	break;
			case 'o':
				if (add) {
				    if (!u->is_oper) {
					u->is_oper = 1;
					IncreaseOpers(u->server);
#ifdef DEBUG
					log("%s is now an IRCop.", u->nick);
#endif
				    }
				} else {
				    if (u->is_oper) {
					u->is_oper = 0;
					me.opers--;
					u->server->opers--;
#ifdef DEBUG
					log("%s is no longer an IRCop.", u->nick);
#endif
				    }
				}
				break;
#ifdef IRCD_2_10
			/* IRCnet propegates AWAY by setting usermode +a */
			case 'a':
				if (add) {
				    if (!u->is_away) {
					u->is_away = 1;
					me.away++;
				    }
				} else {
				    if (u->is_away) {
					u->is_away = 0;
					me.away--;
				    }
				}
				break;
#endif
			default:
				break;
		}
		modes++;
	}
}

static Server *new_server(char *name, char *desc)
{
	Server *s;

	s = calloc(sizeof(Server), 1);
	if (!name)
		name = "";
	memcpy(s->name, name, MAXHOST);
	if (!desc)
		desc = "";
	memcpy(s->desc, desc, LINE);
	add_server_to_hash_table(name, s);

#ifdef DEBUG
	log("new_server: %s, %s", name, desc);
#endif

	return s;
}

#ifdef IRCD_2_10
void AddServer(char *name, int hops, int token, char *desc, char* uplink)
#else
void AddServer(char *name, int hops, char *desc, char* uplink)
#endif
{
	Server *s;

#ifndef FNMATCH
	if (strchr(name, '*')) {
		globops(me.name, "FATAL ERROR: FNMATCH must be #define'ed to process server: %s", name);
		log("FATAL ERROR: FNMATCH must be #define'ed to process server: %s", name);
		SendQuit(finduser(s_StatServ), 0, "FATAL ERROR");
		sleep(1);
		close(servsock);
		SaveStats();
		exit(0);
	}
#endif

#ifdef DEBUG
	log("New Server: %s", name);
#endif
	strlower(name);
	s = new_server(name, desc);
	s->hops = hops;
#ifdef IRCD_2_10
	s->token = token;
#endif
	s->connected_since = time(NULL);
	s->last_announce = time(NULL);
	s->ping = -1;
	s->lowest_ping = 0;
	s->highest_ping = 0;
	s->t_lowest_ping = time(NULL);
	s->t_highest_ping = time(NULL);
        if (uplink == NULL)
                s->uplink = NULL;
        else
                s->uplink = findserver(uplink);
	IncreaseServers();
	AddStats(s);
}

void DelServer(char *name)
{
	Server *s = findserver(name);

#if defined(IRCD_HYB6) || defined(IRCD_2_10)
	/* This is to support the QS capab
	** IRCnet 2.10.x does this too
	*/
	Server *up = NULL;
	User* u = NULL;
	int i;
	for (i=0; i<U_TABLE_SIZE; i++) {
		u=userlist[i];
		if (u) {
			if (!strcmp(name, u->server->name)) {
				DelUser(u->nick);
			}
		}
	}
	for (i=0; i<S_TABLE_SIZE; i++) {
		up=serverlist[i];
		if (up) {
			if (up->uplink) {
				if (!strcmp(name, up->uplink->name)) {
					DelServer(up->name);
				}
			}
		}
	}
#endif

	if (!s || !name) {
		log("DelServer(): %s failed!", name);
		return;
	}

	del_server_from_hash_table(name, s);
	s->stats->lastseen = time(NULL);
	DecreaseServers();
	free(s);
}

#ifdef IRCD_2_10
Server *findservtoken(int token)
{
	Server *s;
	register int i;

        for(i=0; i<S_TABLE_SIZE; i++) {
                s = serverlist[i];
                if (s) {
                        if (s->token == token) {
                                return s;
                        }
                }
        }

/* We should never get here */
#ifdef DEBUG
        log("findservtoken(%d) -> %s", token, (s) ? s->name : "NOT FOUND");
#endif

/* I'd rather return an explicit NULL here */
        return NULL;
}
#endif

Server *findserver(char *name)
{
	Server *s;
#ifdef FNMATCH
	int i;
#endif

	strlower(name); 
#ifdef DEBUG
	log("findserver(%s)", name);
#endif
	s = serverlist[HASH(name, S_TABLE_SIZE)];
	while (s && strcasecmp(s->name, name) != 0)
		s = s->next;
	if (s)
		return s;

#ifdef FNMATCH
/* The name in the server table is the pattern (with the *) 
** The name is what is passed into this function (usually full 
** server name)
**
** NOTE:  This also is a performance hit, if there are a number of masked
**        servers or there are a lot of users on that masked server/set of
**        servers hidden by the mask
*/
	for(i=0; i<S_TABLE_SIZE; i++) {
		s = serverlist[i];
		if (s) {
			if (!fnmatch(s->name, name, 0)) {
				return s;
			}
		}
	}
#endif /* FNMATCH */

/* We should never get here unless the server doesn't exist */
#ifdef DEBUG
	log("findserver(%s) -> %s", name, (s) ? s->name : "NOT FOUND");
#endif

/* I'd rather return an explicit NULL here */
	return NULL;
}

void init_server_hash()
{
	int i;
	Server *s, *b;

	for (i = 0; i < S_TABLE_SIZE; i++) {
		s = serverlist[i];
		while (s) {
			b = s->next;
#ifdef DEBUG
	log("init_server_hash() deleting %s", s->name);
#endif
			free(s);
			s = b;
		}
		serverlist[i] = NULL;
	}
	bzero((char *)serverlist, sizeof(serverlist));
#ifdef IRCD_2_10
	AddServer(me.name, 0, 0, me.desc, NULL);
#else
	AddServer(me.name, 0, me.desc, NULL);
#endif
}

void LoadMyUsers()
{
	FILE *fp;
	char buf[BUFSIZE];
	char *user, *pass, *time;
	MyUser *myu;

	if ((fp = fopen(USERFILE, "r")) == NULL) {
		fprintf(stderr, "Please type this: echo \"<nickname> <password> 0 >>"
		"%s\"\n", USERFILE);
		fprintf(stderr, "Replace nickname with your nickname, and password"
		" with your password, encrypting if necessary\n");
		fprintf(stderr, "This is used for the LOGIN feature of Stats.\n");
		log("Unable to open userlist.");
		exit(0);
		myuhead = NULL;
	}

	while (fgets(buf, BUFSIZE, fp)) {
		user = strtok(buf, " ");
		pass = strtok(NULL, " ");
		time = strtok(NULL, "\n");

		if ((!user) || (!pass)) {
			fprintf(stderr, "Invalid entry in user file: %s\n",
				USERFILE);
			fprintf(stderr, "Format: nick pass 0\n");
			fprintf(stderr, "Or: nick pass <time last seen>\n");
			fprintf(stderr, "Exiting...\n");
			exit(0);
		}

                myu = smalloc(sizeof(MyUser));
                myu->username = strdup(user);
                myu->password = strdup(pass);
                myu->lastseen = atol(time);
                myu->ison = 0;

		if (!time) {
			log("Invalid time value in user file %s, defaulting to 0", USERFILE);
			myu->lastseen = 0;
		} else {
			myu->lastseen = atol(time);
		}

		if (!myuhead) {
			myuhead = myu;
			myuhead->next = NULL;
		} else {
			myu->next = myuhead;
			myuhead = myu;
		}
	}
	fclose(fp);
}

void SaveMyUsers()
{
	MyUser *myu;
	FILE *fp;

	if (!myuhead)
		return;

	if ((fp = fopen(USERFILE, "w")) == NULL) {
		log("Unable to open %s for writing.", USERFILE);
		return;
	}
	for (myu = myuhead; myu; myu = myu->next) {
		fprintf(fp, "%s %s %ld\n", myu->username, myu->password,
			myu->lastseen);
	}
	fclose(fp);
}

void DeleteMyUser(char *username)
{
	MyUser *myu, *prev = NULL;

	if (!myuhead)
		return;

	myu = myuhead;

	while (myu) {
		if (!strcasecmp(myu->username, username))
			break;
		prev = myu;
		myu = myu->next;
	}
	if (!myu)
		return;

	if (!prev) {
		myuhead = myu->next;
	} else if (myu->next == NULL) {
		prev->next = NULL;
	} else {
		prev->next = myu->next;
	}
	free(myu->username);
	free(myu->password);
	free(myu);
}

MyUser *findmyuser(char *nick)
{
	MyUser *myu;

	for (myu = myuhead; myu; myu = myu->next) {
		if (!strcasecmp(nick, myu->username))
			return myu;
	}
	return NULL;
}
